#include <utility>
#include <algorithm>
#include <vector>
#include <sstream>
#include <iostream>
#include <queue>
#include <string>
#include <functional>
#include <algorithm>
#include "./Timer.h"
#include "./DependentDijkstra.h"

using std::pair;
using std::cout;
using std::endl;
// _____________________________________________________________________________
// CLASS: TransitDependentDijkstra
// Dijkstra algorithm for transitnetwork based on dependent graph
DependentDijkstra::DependentDijkstra(
                                TransitNetworkDependent* roadNetwork)
{
  this->_tranGraph = roadNetwork;
}
// _____________________________________________________________________________
double DependentDijkstra::computeShortestPath(int startId,
                                            Time startTime,
                                            int endId, bool buildPath)
{
  std::priority_queue <pair<int, int>,
                  std::vector<pair<int, int> >,
                  std::greater<pair<int, int > > > _openNodeQueue;

  if (startId == -1)
  {
    printf("start stop is not known\n");
    return -1;
  }

  if (endId == -1)
  {
    printf("end stop is not known\n");
    return -1;
  }
  int transferTime = 4 * 60;
  _endStopFound = false;
  exploredNodes = 0;
  labelSets.clear();
  labelSets.resize(_tranGraph->getNumNodes(), LabelSet(&labels, 4 * 60));
  int currentId = startId;
  int currentWight;
  int currentLabelId;
  Connection currentStopTime(startTime,
                           startTime,-1);
  vector <int> currentCriterias;
  currentCriterias.push_back(0);
  currentCriterias.push_back(0);
  currentCriterias.push_back(0);
  bool flag = labelSets[currentId].AddLabel(currentStopTime, currentCriterias,
                                   startId, -1, UNKNOWN, false, false, true, false);
  assert(flag);
  _openNodeQueue.push(std::pair<int, int> (0, labels.size()-1));
  vector<EdgeDependent>* edges;

  int newId;
  bool dijkstraFinished = false;
  // get straight line distance to endnode
  size_t gepopt = 0;

  while (!dijkstraFinished)
  {
    // get minimum of node queue
    currentLabelId = _openNodeQueue.top().second;
    currentId = labels[currentLabelId].NodeId;
    currentStopTime = labels[currentLabelId].connection;
    currentWight = _openNodeQueue.top().first;
//    cout << labels[currentLabelId].expandedFromLabelId << " "<< currentId << " " <<currentStopTime.lineId << " " << labels[currentLabelId].criterias[0]<<endl;
//    currentStopTime.departureTime.printTime();
//    currentStopTime.arrivalTime.printTime();
    _openNodeQueue.pop();
    gepopt++;
    // check if already explored
    if (labels[currentLabelId].removed == true)
    {
       dijkstraFinished = _openNodeQueue.empty();
       continue;
    }
    // get outgoing edges from this node
    edges = _tranGraph->getOutEdges(currentId);
    // compute all new distances in loop
    for (size_t i = 0; i < edges->size(); i++)
      if (labels[currentLabelId].expandedFromLabelId == -1
              || ((*edges)[i].otherNodeId != labels[labels[currentLabelId]
                                                    .expandedFromLabelId].NodeId
              && (*edges)[i].otherNodeId != currentId))
      {
        // get all new distances
        EdgeDependent& newEdge = (*edges)[i];
        newId = newEdge.otherNodeId;
        vector <Connection> newStopTimes;
        bool domination = false;
        newEdge.GetWight(currentStopTime, transferTime,
                &newStopTimes, currentId == startId, false, &domination);
        if (newStopTimes.size() == 0)
          continue;
        //if (newId == 30) printf("gleich!\n");
        for (size_t i = 0; i < newStopTimes.size(); i++)
        {
          vector <int> newCriterias = labels[currentLabelId].criterias;

          newCriterias[0] = currentWight
                  + newStopTimes[i]
                  .arrivalTime.minusInSecondsDependent(currentStopTime.arrivalTime);
          if (newStopTimes[i].lineId != currentStopTime.lineId)
            newCriterias[2] += 1;
          if (labelSets[newId].AddLabel(newStopTimes[i], newCriterias,
                  newId, currentLabelId, UNKNOWN, false, false, true, false))
          {
//            cout << "added: " << newId << " " << newStopTimes[i].lineId << " " << newCriterias[0] / 60 << endl;
//            newStopTimes[i].departureTime.printTime();
//            newStopTimes[i].arrivalTime.printTime();
            _openNodeQueue.push(std::pair<int, int> (newCriterias[0], labels.size() - 1));
          }
        }
      }
    exploredNodes++;
    // for the rest of explorations to 4 min
    dijkstraFinished = _openNodeQueue.empty() || currentId == endId;
  }
  // reset all ndos left in queue, empty queue this way...
  printf("explored: %zu\n", exploredNodes);
  printf("popped  : %zu\n", gepopt);
  if (labelSets[endId]._expandedFrom.size() == 0)
  {
    transfers=0;
    time=0;
    //printf("shortest Path not found from %d to %d\n", startId, endId);
     if (buildPath)
       transitShortestPath.clear();
    return -1;
  }
  size_t k = (int)labelSets[endId]._expandedFrom[0][0];
  transfers = labels[k].criterias[2];
  time = labels[k].criterias[0];
  if (buildPath)
    reconstructShortestPath(startId, endId);
  return 0;
}
// _____________________________________________________________________________
void DependentDijkstra::reconstructShortestPath(size_t startId, size_t endId)
{
  transitShortestPath.clear();
  size_t currentId = endId;
  size_t currentLabelId = (int)labelSets[currentId]._expandedFrom[0][0];
  size_t oldLabelId;
  size_t oldId;
  bool flag = true;
  while (flag)
  {
    string line;
    std::stringstream ss (std::stringstream::in | std::stringstream::out);
    if (currentId == endId)
      ss << _tranGraph->getStopName(currentId)
         << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
         << "\t\t\tTripId:" << labels[currentLabelId].connection.lineId;
    else if (currentId != startId)
      ss << _tranGraph->getStopName(currentId)
         << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
         << "\tDEP:" << (int)labels[oldLabelId].connection.departureTime.hour()
         << ":" << (int)labels[oldLabelId].connection.departureTime.minute()
         << ":" << (int)labels[oldLabelId].connection.departureTime.second()
         << "\tTripId:" << labels[currentLabelId].connection.lineId;
    else
      ss << _tranGraph->getStopName(currentId)
         << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
         << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
         << "\tDEP:" << (int)labels[oldLabelId].connection.departureTime.hour()
         << ":" << (int)labels[oldLabelId].connection.departureTime.minute()
         << ":" << (int)labels[oldLabelId].connection.departureTime.second();
    oldId = currentId;
    oldLabelId = currentLabelId;
    flag = currentId != startId;
    if (flag)
    {
      currentLabelId = labels[currentLabelId].expandedFromLabelId;
      currentId = labels[currentLabelId].NodeId;
    }
    //cout << "startid=" << startId << " endId=" << endId<< " curId=" << currentId<<endl;
    line = ss.str();
    transitShortestPath.push_back(line);
  }
}