/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

#include <iostream>
#include <assert.h>
#include <time.h>
#include "./EdgeDependent.h"
#include "./Time.h"
#include <limits>
#include <bits/stl_vector.h>
using std::vector;
using std::cout;
using std::endl;
using std::unordered_map;
using std::unordered_set;
// _____________________________________________________________________________
EdgeDependent::EdgeDependent(int otherNodeId)
{
  this->otherNodeId = otherNodeId;
  _times.clear();
  //this->_times = new vector<StopTime>();
}
// _____________________________________________________________________________
void EdgeDependent::BuildTripsMap()
{
  _tripIdsToConections.clear();
  for (size_t i = 0; i < _times.size(); i++)
  {
    int lineId = _times[i].lineId;
    if (_tripIdsToConections.count(lineId) == 0)
    {
      vector <int> buf;
      // Before: buf.push_back(lineId); ...
      buf.push_back(i);
      _tripIdsToConections.insert(std::pair <int, vector<int> > (lineId, buf));
    } else
    {
      _tripIdsToConections[lineId].push_back(i);
    }
  }
}
// _____________________________________________________________________________
void EdgeDependent::AddStopTimeToTheTrip(int timeId,
                                         int lineId)
{
  if (_tripIdsToConections.count(lineId) == 0)
  {
    vector <int> buf;
    // Before: buf.push_back(lineId); ...
    buf.push_back(timeId);
    _tripIdsToConections.insert(std::pair <int, vector<int> > (lineId, buf));
  }
  else
  {
    size_t i = 0;
    while ((i < _tripIdsToConections[lineId].size())
    && (_times[_tripIdsToConections[lineId][i]].departureTime <
        _times[timeId].departureTime))
      i++;

    if (i >= _tripIdsToConections[lineId].size())
    {
      _tripIdsToConections[lineId].push_back(timeId);
    } else
    {
      vector<int>::iterator it = _tripIdsToConections[lineId].begin();
      _tripIdsToConections[lineId].insert(it + i, timeId);
    }
  }
}
// _____________________________________________________________________________
void EdgeDependent::AddStopTime(Time startTime,
                                Time endTime,
                                int lineId)
{
  Connection stopTime(startTime, endTime, lineId);
  size_t i = 0;
  while ((i < _times.size()) && (_times[i].departureTime < startTime)) i++;
  //if (lineId == 1 && otherNodeId== 2)
  //  cout << i << " " << "T:"<<(int)startTime.hour() << ",H:" << (int)startTime.minutes() << ",M:" << (int)startTime.seconds() << ",id="<<lineId << endl;
  if (i < _times.size())
    if ( (_times[i].departureTime == startTime) && lineId == _times[i].lineId)
    {
      //cout << endl <<"ALARMM!!! Double StopTime!!!"<< endl;
      //cout << i << " " << "T:"<<(int)_times[i].startTime.weekday() << ",H:" << (int)_times[i].startTime.hour() << ",M:" << (int)_times[i].startTime.minutes() << ",id="<<_times[i].lineId << endl;
      //cout << i << " " << "T:"<<(int)startTime.weekday() << ",H:" << (int)startTime.hour() << ",M:" << (int)startTime.minutes() << ",id="<<lineId << endl;
      return;
    }
  if (i >= _times.size())
  {
    _times.push_back(stopTime);
  }
  else
  {
    vector<Connection>::iterator it=_times.begin();
    _times.insert(it + i, stopTime);
  }

  //AddStopTimeToTheTrip(i, lineId);
  //if (lineId==4061||lineId==26171)
  //for (int j=0; j< _times.size(); j++)
  //  if (_times[j].startTime.weekday()==0&&_times[j].startTime.hour()==9&&otherNodeId==587)
  //  cout << j << " " << "T:"<<(int)_times[j].startTime.weekday() << ",H:" << (int)_times[j].startTime.hour() << ",M:" << (int)_times[j].startTime.minutes() << ",id="<<_times[j].lineId << endl;
}
// _____________________________________________________________________________
Connection::Connection()
{
}
// _____________________________________________________________________________
Connection::Connection(Time startTime,
                   Time endTime,
                   int lineId)
{
      this->departureTime = startTime;
      this->arrivalTime = endTime;
      this->lineId = lineId;
}
// _____________________________________________________________________________

Connection::Connection(int lineId)
{
      this->lineId = lineId;
}
// _____________________________________________________________________________
void EdgeDependent::GetNextConectionOfAllTrips(const Connection& inTime,
                                               vector<Connection>* stopTimesOut)
{
  for (unordered_map <int, vector<int> >::iterator it =
       _tripIdsToConections.begin();
       it != _tripIdsToConections.end();
       ++it)
  {
    int lineId = it->first;
    stopTimesOut->push_back(_times[findFirstDepartureTimeIndexForTrip(lineId,
                                                          inTime.arrivalTime)]);
  }

}
// _____________________________________________________________________________
void EdgeDependent::GetWight(const Connection& inTime, int transferTime,
                             vector<Connection>* stopTimesOut,
                             bool ifStartNode, bool isMultictriteria,
                             bool* domination)
{
  int bufTransferTime = transferTime;
  if (ifStartNode)
    transferTime = 0;
  if (_times.size() == 0) return;
  // Find with binary serch first StopTime, that can maybe be used
  if (_tripIdsToConections.count(inTime.lineId) == 0 && isMultictriteria)
  {
    stopTimesOut->clear();
    Connection buf(inTime.departureTime, inTime.arrivalTime, inTime.lineId);
    if (!ifStartNode)
    {
      buf.arrivalTime.addSeconds(4 * 60);
    }
    GetNextConectionOfAllTrips(buf, stopTimesOut);
    *domination = false;
    return;
  }
  else if (_tripIdsToConections.count(inTime.lineId) != 0 )
  {
    size_t sameTrip = findFirstDepartureTimeIndexForTrip(inTime.lineId, inTime.arrivalTime);
    stopTimesOut->push_back(_times[sameTrip]);
  }
  size_t minCenter = findFirstDepartureTimeIndex(inTime.arrivalTime);
  size_t center = minCenter;
  int deffToDep = 0;
  int deffToArr = 0;

  int minDeffToArr = _times[center].arrivalTime.minusInSecondsDependent(inTime.arrivalTime);
  size_t minPos = center;

  // Search for next best train conection
  // find such StopTime, that we have enough time to transfer (if needed)
  // and can arrive as soon as possible by the EndStation of this edge.
  // At the end in _times by postion minPos is best StopTime,
  // wich will be returned
  size_t counter = 0;
  center ++;
  if (center >=_times.size())
    center = 0;
  while (counter < _times.size() && minDeffToArr > deffToDep)
  {
    deffToArr = _times[center].arrivalTime.minusInSecondsDependent(inTime.arrivalTime);
    deffToDep = _times[center].departureTime.minusInSecondsDependent(inTime.arrivalTime);
    if ((deffToDep >= transferTime ||_times[center].lineId == inTime.lineId)
     && (deffToArr < minDeffToArr))
    {
      minDeffToArr = _times[center].arrivalTime.minusInSecondsDependent(inTime.arrivalTime);
      minPos = center;
    }
    counter++;
    center ++;
    if (center == _times.size()) center = 0;
  }


  transferTime = bufTransferTime;

  if (_times[minPos].lineId != inTime.lineId)
  {
    stopTimesOut->push_back(_times[minPos]);
  }

  center = minCenter;
  counter = 0;
  deffToDep = _times[center].departureTime.minusInSecondsDependent(inTime.arrivalTime);

  while (counter < _times.size() && (minDeffToArr + transferTime) > deffToDep)
  {
    deffToArr = _times[center].arrivalTime.minusInSecondsDependent(inTime.arrivalTime);
    deffToDep = _times[center].departureTime.minusInSecondsDependent(inTime.arrivalTime);
    if ((center != minPos) && ((minDeffToArr + transferTime) > deffToArr)
     && (deffToDep >= transferTime ||_times[center].lineId == inTime.lineId)
     && _times[center].lineId != _times[minPos].lineId
     && _times[center].lineId != inTime.lineId)
    {
      stopTimesOut->push_back(_times[center]);
    }

    counter++;
    center++;
    if (center == _times.size()) center = 0;
  }
}


size_t EdgeDependent::findFirstDepartureTimeIndex(const Time& startTime)
{

  assert(_times.size() > 0);
  size_t leftBorder = 0;
  size_t rightBorder = _times.size();
  // if number of elements is even, then center is the element more
  // to the right! for example in 1, 4, 5, 16 the center would be 5
  size_t center = (rightBorder + leftBorder) / 2;

  // Find with binary search first StopTime, that can possibly be used

  // check if requested starttime is outside of the known departure times
  if ((_times[_times.size() - 1].departureTime < startTime)
      || startTime < _times[0].departureTime)
  {
    center = 0;
    rightBorder = 0;
  }
  // check if its actually same as first departure time
  else if (_times[0].departureTime == startTime)
  {
    center = 0;
    rightBorder = 0;
  }
  else if (_times.back().departureTime == startTime)
  {
    center = _times.size() - 1;
    leftBorder = center;
  }
  else
  {
    // time should bet inside the time vector
    assert(_times[0].departureTime < startTime &&
           startTime < _times.back().departureTime);
    while (center > leftBorder)
    {
      if (_times[center].departureTime < startTime)
        leftBorder = center;
      else
        rightBorder = center;
      center = (rightBorder + leftBorder) / 2;
    }
  }
  assert(center == leftBorder);
  // now center is definitely before or at
  // the first possible start time, but it may be that rightborder
  // is the first possible start time, not center...
  if ((_times[center].departureTime < startTime)
   && rightBorder != _times.size())
    center = rightBorder;
  return center;
}

size_t EdgeDependent::findFirstDepartureTimeIndexForTrip(int lineId,
                                                         const Time& startTime)
{
  assert(_tripIdsToConections[lineId].size() > 0);
  size_t leftBorder = 0;
  size_t rightBorder = _tripIdsToConections[lineId].size();
  // if number of elements is even, then center is the element more
  // to the right! for example in 1, 4, 5, 16 the center would be 5
  size_t center = (rightBorder + leftBorder) / 2;

  // Find with binary search first StopTime, that can possibly be used

  // check if requested starttime is outside of the known departure times
  if ((_times[_tripIdsToConections[lineId].back()].departureTime < startTime)
      || startTime < _times[_tripIdsToConections[lineId][0]].departureTime)
  {
    center = 0;
    rightBorder = 0;
  }
  // check if its actually same as first departure time
  else if (_times[_tripIdsToConections[lineId][0]].departureTime == startTime)
  {
    center = 0;
    rightBorder = 0;
  }
  // check if its same as last departure time
  else if (_times[_tripIdsToConections[lineId].back()].departureTime == startTime)
  {
    center = _tripIdsToConections[lineId].size() - 1;
    leftBorder = center;
    rightBorder = center;
  }
  else
  {
    // starttime should be inside the time vector
    assert(_times[_tripIdsToConections[lineId][0]].departureTime < startTime &&
           startTime < _times[_tripIdsToConections[lineId].back()].departureTime);
    while (center > leftBorder)
    {
      if (_times[_tripIdsToConections[lineId][center]].departureTime < startTime)
        leftBorder = center;
      else
        rightBorder = center;
      center = (rightBorder + leftBorder) / 2;
    }
  }
  assert(center == leftBorder);
  assert(rightBorder == center || rightBorder == center + 1);
  // now center is definitely before or at
  // the first possible start time, but it may be that rightborder
  // is the first possible start time, not center...
  if ((_times[_tripIdsToConections[lineId][center]].departureTime < startTime)
   && rightBorder < _tripIdsToConections[lineId].size())
    center = rightBorder;
  return _tripIdsToConections[lineId][center];
}