/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

#ifndef ROUTEPLANNINGV2_EDGEDEPENDENT_H_
#define ROUTEPLANNINGV2_EDGEDEPENDENT_H_

#include <vector>
#include <unordered_map>
#include <unordered_set>
#include "./Time.h"
using std::vector;
using std::unordered_map;
// Object, which contains all information about one conection at a certain time
// between two naighbouring stations
class Connection
{
 public:
  // Deprture time from Station A
  Time departureTime;
  // Arrival time to Station B
  Time arrivalTime;
  // corresponding Id of a line, wich this conection use
  int lineId;
  Connection(Time startTime, Time endTime, int lineId);
  Connection(int lineId);
  Connection();
};

class EdgeDependent
{
 public:
  EdgeDependent(int otherNodeId);
  // Add StopTime to the this edge
  void AddStopTime(Time startTime, Time endTime, int lineId);
  void BuildTripsMap();
  void AddStopTimeToTheTrip(int timeId, int lineId);
  // Find corresponding Connections to given Connection,
  // where inTime is a time at wich a "train"
  // arrived the start node of this edge
  // TODO(Robin): remove multicriteria parameter, it is not used(?)
  void GetWight(const Connection& inTime, int transferTime,
                std::vector<Connection>* stopTimesOut,
                bool ifStartNode, bool isMultictriteria, bool* domination);
  void GetNextConectionOfAllTrips(const Connection& inTime,
                                  std::vector<Connection>* stopTimesOut);
  int otherNodeId;
  // StopTimes, wich belongs to the edge, sorted ascending by time
  // Weekday:hour:minutes
  vector<Connection> _times;
  unordered_map <int, vector<int> > _tripIdsToConections;
private:
  size_t findFirstDepartureTimeIndex(const Time& startTime);
  size_t findFirstDepartureTimeIndexForTrip(int lineId, const Time& startTime);
};

#endif  // ROUTEPLANNINGV2_EDGEDEPENDENT_H_
