
#include <gtest/gtest.h>
#include <vector>
#include "./EdgeDependent.h"

using std::vector;

TEST(EdgeDependentTest, getNextConnectionOfAllTrips)
{
  int dummyNodeId = -1;
  int dummyTripId = -1;
  EdgeDependent testEdge(dummyNodeId);
  Time dummyDepartureTime(0, 0 , 0);
  // First Test: Test finding a connection shortly after or at the same time
  // as the start time
  int firstLineId = 1;
  int secondLineId = 2;
  testEdge.AddStopTime(Time(5, 30, 2), Time(5, 32, 3), firstLineId);
  testEdge.BuildTripsMap();
  vector<Connection> outTimes;
  Time arrivalTime(5, 30, 0);
  Connection incomingConnection(dummyDepartureTime, arrivalTime, dummyTripId);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);

  // Test after this time should still find same one
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 35);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);

  
  // second test with antoher stop time but same results
  testEdge.AddStopTime(Time (5, 38, 3), Time(5, 38, 50), firstLineId);
testEdge.BuildTripsMap();
  
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 2);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);

  
  // second test just after first starttime
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 38, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[0].arrivalTime);

  // having to go through the whole day, then start at next day...
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 38, 4);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);


  // add stop to same time as first line
  testEdge.AddStopTime(Time (5, 38, 3), Time(5, 38, 50), secondLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 38, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(2, outTimes.size());
  ASSERT_EQ(Time(5, 38, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[0].arrivalTime);
  ASSERT_EQ(Time(5, 38, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[1].arrivalTime);
  // lines should be different, one should be first and one should be second
  ASSERT_TRUE(outTimes[0].lineId == firstLineId ||
              outTimes[0].lineId == secondLineId);
  ASSERT_TRUE(outTimes[1].lineId == firstLineId ||
              outTimes[1].lineId == secondLineId);
  ASSERT_TRUE(outTimes[0].lineId != outTimes[2].lineId);

  // add another stop to second line add it inbetween the stops
  testEdge.AddStopTime(Time (5, 32, 5), Time(5, 32, 17), secondLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 32, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(2, outTimes.size());
  // now sorted by line id...
  ASSERT_EQ(Time(5, 38, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[0].arrivalTime);
  ASSERT_EQ(Time(5, 32, 5), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 32, 17), outTimes[1].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);

  int thirdLineId = 3;
  testEdge.AddStopTime(Time (23, 59, 58), Time(0, 1, 19), thirdLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(23, 59, 59);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(3, outTimes.size());
  ASSERT_EQ(outTimes[0].lineId, firstLineId);
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);
  ASSERT_EQ(outTimes[1].lineId, secondLineId);
  ASSERT_EQ(Time(5, 32, 5), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 32, 17), outTimes[1].arrivalTime);
  ASSERT_EQ(outTimes[2].lineId, thirdLineId);
  ASSERT_EQ(Time(23, 59, 58), outTimes[2].departureTime);
  ASSERT_EQ(Time(0, 1, 19), outTimes[2].arrivalTime);
}



TEST(EdgeDependentTest, getWeight)
{
  int transferTime = 4 * 60;  // 4 Minutes
  bool isStartNode = false;
  bool isMultiCriteria = false;
  bool dummyDomination = false;
  vector<Connection> outTimes;
  // Test to get stoptime that starts later but arrives earlier
  // (from same trip)?
  int dummyNodeId = -1;
  int dummyTripId = -1;
  EdgeDependent testEdge(dummyNodeId);
  Time dummyArrivalTime(0, 0, 0);
  Time dummyDepartureTime(0, 0, 0);
  Connection incomingConnection(dummyDepartureTime, dummyArrivalTime,
                                dummyTripId);
  int firstLineId = 1;
  int secondLineId = 2;
  int thirdLineId = 3;
  int fourthLineId = 4;

  // Simple test, just get first stopstime
  testEdge.AddStopTime(Time(13, 30, 2), Time(13, 32, 3), firstLineId);
  testEdge.BuildTripsMap();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  // Test to get first stoptime while arriving at exactly the same time
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 2);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  // Test to get first stoptime while arriving after the departure time
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 10);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);

  // Test to get connections that arrive at target station
  // more than one day later (one day after starttime + transfertime)!
  // Test to get first stoptime while arriving with different line
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = secondLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);


  
  // Test to get first stoptime while arriving with different line
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 38, 0);
  incomingConnection.lineId = secondLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);

  // Test to get one stoptime even though other one is within transfer time
  // but from same trip!
  testEdge.AddStopTime(Time(13, 30, 3), Time(13, 32, 4), firstLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);

  // Same test arriving with different line
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 38, 0);
  incomingConnection.lineId = secondLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  
  // Test to get two stop times, one within transfer time, from different trip
  testEdge.AddStopTime(Time(13, 34, 1), Time(13, 36, 2), secondLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(2, outTimes.size());
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  ASSERT_EQ(Time(13, 34, 1), outTimes[1].departureTime);
  ASSERT_EQ(Time(13, 36, 2), outTimes[1].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);

  // Same tests, but stop times are outside of transfer time at departing staiton
  testEdge.AddStopTime(Time(13, 35, 8), Time(13, 36, 1), thirdLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(3, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  ASSERT_EQ(Time(13, 34, 1), outTimes[1].departureTime);
  ASSERT_EQ(Time(13, 36, 2), outTimes[1].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);
  ASSERT_EQ(Time(13, 35, 8), outTimes[2].departureTime);
  ASSERT_EQ(Time(13, 36, 1), outTimes[2].arrivalTime);
  ASSERT_EQ(thirdLineId, outTimes[2].lineId);
  // Test to have connection that is only inside transfer time 
  // at arrival station and should therefore not be found!
  testEdge.AddStopTime(Time(13, 32, 2), Time(13, 36, 4), fourthLineId);
  testEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(13, 30, 0);
  incomingConnection.lineId = firstLineId;
  testEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(3, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...
  ASSERT_EQ(Time(13, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(13, 32, 3), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  ASSERT_EQ(Time(13, 34, 1), outTimes[1].departureTime);
  ASSERT_EQ(Time(13, 36, 2), outTimes[1].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);
  ASSERT_EQ(Time(13, 35, 8), outTimes[2].departureTime);
  ASSERT_EQ(Time(13, 36, 1), outTimes[2].arrivalTime);
  ASSERT_EQ(thirdLineId, outTimes[2].lineId);

  // Test where you should take second trip even though it starts a lot later
  // because it arrives within transfer time at next station
  EdgeDependent thirdTestEdge(dummyNodeId);
  thirdTestEdge.AddStopTime(Time(15, 2, 3), Time(15, 8, 4), firstLineId);
  thirdTestEdge.AddStopTime(Time(15, 7, 3), Time(15, 12, 3), secondLineId);
  thirdTestEdge.BuildTripsMap();
  incomingConnection.arrivalTime = Time(15, 0, 0);
  incomingConnection.lineId = firstLineId;
  outTimes.clear();
  printf("1\n");
  thirdTestEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(2, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...
  ASSERT_EQ(Time(15, 2, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(15, 8, 4), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  ASSERT_EQ(Time(15, 7, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(15, 12, 3), outTimes[1].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);
  // Test where you should take second trip even though it starts a lot later
  // because it arrives within transfer time at next station
  EdgeDependent fourthTestEdge(dummyNodeId);
  fourthTestEdge.AddStopTime(Time(23, 50, 3), Time(23, 57, 5), secondLineId);
  fourthTestEdge.AddStopTime(Time(0, 0, 3), Time(0, 0, 4), thirdLineId);
  fourthTestEdge.BuildTripsMap();
  incomingConnection.arrivalTime = Time(23, 46, 3);
  incomingConnection.lineId = firstLineId;
  outTimes.clear();
  fourthTestEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  //ASSERT_EQ(2, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...
  ASSERT_EQ(Time(23, 50, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(23, 57, 5), outTimes[0].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[0].lineId);
  ASSERT_EQ(Time(0, 0, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(0, 0, 4), outTimes[1].arrivalTime);
  ASSERT_EQ(thirdLineId, outTimes[1].lineId);

  // Test where one is within transfer time after day boundary
  // Test to have connection that is only inside transfer time
  // at arrival station and should therefore not be found!
  EdgeDependent secondTestEdge(dummyNodeId);
  secondTestEdge.AddStopTime(Time(0, 0, 3), Time(0, 1, 4), firstLineId);
  secondTestEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(23, 59, 59);
  incomingConnection.lineId = firstLineId;
  secondTestEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(0, 0, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(0, 1, 4), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);

  // Same Test but arriving with different line
  outTimes.clear();
  incomingConnection.arrivalTime = Time(23, 56, 3);
  incomingConnection.lineId = secondLineId;
  secondTestEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  ASSERT_EQ(1, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...
  ASSERT_EQ(Time(0, 0, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(0, 1, 4), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);

  // Test finding first trip one same day second on next day but within transfer
  // time
  secondTestEdge.AddStopTime(Time(23, 50, 3), Time(23, 57, 5), secondLineId);
  secondTestEdge.BuildTripsMap();
  outTimes.clear();
  incomingConnection.arrivalTime = Time(23, 46, 3);
  incomingConnection.lineId = firstLineId;
  secondTestEdge.GetWight(incomingConnection, transferTime, &outTimes,
                    isStartNode, isMultiCriteria, &dummyDomination);
  //ASSERT_EQ(2, outTimes.size());
  // WARNING: Test assumes certain order of results that need not be guaranteed
  // and might change...


  ASSERT_EQ(Time(23, 50, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(23, 57, 5), outTimes[1].arrivalTime);
  ASSERT_EQ(secondLineId, outTimes[1].lineId);

  ASSERT_EQ(Time(0, 0, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(0, 1, 4), outTimes[0].arrivalTime);
  ASSERT_EQ(firstLineId, outTimes[0].lineId);
  // TODO(Robin):
  // Test within transfer time after day boundary, more than one day
  // beyond start time
/*
  // Test after this time should still find same one
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 35);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_TRUE(outTimes[0].departureTime == Time(5, 30, 2));
  ASSERT_TRUE(outTimes[0].arrivalTime == Time(5, 32, 3));


  // second test with antoher stop time but same results
  testEdge.AddStopTime(Time (5, 38, 3), Time(5, 38, 50), firstLineId);


  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 2);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);
   

  // second test just after first starttime
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 30, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 38, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[0].arrivalTime);

  // having to go through the whole day, then start at next day...
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 38, 4);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(1, outTimes.size());
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);


  // add stop to same time as first line
  testEdge.AddStopTime(Time (5, 38, 3), Time(5, 38, 50), secondLineId);
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 38, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(2, outTimes.size());
  ASSERT_EQ(Time(5, 38, 3), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[0].arrivalTime);
  ASSERT_EQ(Time(5, 38, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[1].arrivalTime);
  // lines should be different, one should be first and one should be second
  ASSERT_TRUE(outTimes[0].lineId == firstLineId ||
              outTimes[0].lineId == secondLineId);
  ASSERT_TRUE(outTimes[1].lineId == firstLineId ||
              outTimes[1].lineId == secondLineId);
  ASSERT_TRUE(outTimes[0].lineId != outTimes[2].lineId);

  // add another stop to second line add it inbetween the stops
  testEdge.AddStopTime(Time (5, 32, 5), Time(5, 32, 17), secondLineId);
  outTimes.clear();
  incomingConnection.arrivalTime = Time(5, 32, 3);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(2, outTimes.size());
  ASSERT_EQ(Time(5, 32, 5), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 17), outTimes[0].arrivalTime);
  ASSERT_EQ(Time(5, 38, 3), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 38, 50), outTimes[1].arrivalTime);
  // first comes the second line then first line...
  ASSERT_TRUE(outTimes[0].lineId == secondLineId);
  ASSERT_TRUE(outTimes[1].lineId == firstLineId);

  int thirdLineId = 3;
  testEdge.AddStopTime(Time (23, 59, 58), Time(0, 1, 19), thirdLineId);
  outTimes.clear();
  incomingConnection.arrivalTime = Time(23, 59, 59);
  testEdge.GetNextConectionOfAllTrips(incomingConnection, &outTimes);
  ASSERT_EQ(3, outTimes.size());
  ASSERT_TRUE(outTimes[0].lineId == firstLineId);
  ASSERT_EQ(Time(5, 30, 2), outTimes[0].departureTime);
  ASSERT_EQ(Time(5, 32, 3), outTimes[0].arrivalTime);
  ASSERT_TRUE(outTimes[1].lineId == secondLineId);
  ASSERT_EQ(Time(5, 32, 5), outTimes[1].departureTime);
  ASSERT_EQ(Time(5, 32, 17), outTimes[1].arrivalTime);
  ASSERT_TRUE(outTimes[2].lineId == thirdLineId);
  ASSERT_EQ(Time(23, 59, 58), outTimes[2].departureTime);
  ASSERT_EQ(Time(0, 1, 19), outTimes[2].arrivalTime);*/
}
