/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 * Testclass for Transit network
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gtest/gtest.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unordered_map>
#include <stdlib.h>
#include <string>
#include <utility>
#include <vector>
#include "./CsvParser.h"
#include "./TransitNetworkExpanded.h"
#include "./Dijkstra.h"
#include "./ExpandedDijkstra.h"



TEST(TransitNetworkTest, getStartNodeIds)
{
   // TODO(Robin): check the length that the routeplanner returns is correct
  TransitNetworkExpanded transitNetwork;
  string testDirectory = "./GTFSStartNodeTest";
  int status = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG | S_IROTH |
                                            S_IXOTH);
  if (status != 0)
  {
    printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
    // make test crash if it directory could not be created...
    ASSERT_TRUE(false);
  }
  string calendarFileName = testDirectory +"/calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);

  string tripFileName = testDirectory + "/trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n");
  fclose(tripFile);

  string stopsFileName = testDirectory + "/stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
    "location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U9_U7_1,Berliner Strasse, Zu U9 :),52.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);

  string frequenciesFileName = testDirectory + "/frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:00,27:30:00,10800\n"
    "U7WE,00:00:00,24:00:00,5400\n"
    "U7WD,05:30:00,26:30:00,3600\n"
    "U7FW,05:29:32,25:29:32,7200\n");
  fclose(frequenciesFile);

  string stopTimesFileName = testDirectory + "/stop_times.txt";
  FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
  fprintf(stopTimesFile,
    "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
    "pickup_type,dropoff_type\n"
    "U9FW,5:30:0,5:32:17,U9_U7_1,1,0,0,0\n"
    "U9FW,5:34:13,05:38:0,U92,2,0,0,0\n"
    // last stop transfer and departure time equal if transfer time = 4 minutes
    "U7WE,0:0:0,0:1:0,U71,1,0,0,0\n"
    "U7WE,0:4:0,0:6:0,U9_U7_1,2,0,0,0\n"
    "U7WD,5:30:0,05:31:0,U70,1,0,0,0\n"
    "U7WD,5:32:17,05:33:6,U71,2,0,0,0\n"
    "U7WD,5:34:05,05:35:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:29:32,5:30:0,U70,1,0,0,0\n"
    "U7FW,5:32:0,5:33:0,U71,2,0,0,0\n"
    "U7FW,5:34:0,5:35:09,U9_U7_1,3,0,0,0\n"
    "U7FW,5:36:07,5:37:0,U73,4,0,0,0\n");

  fclose(stopTimesFile);
  int transferTimeInSeconds = 4 * 60;
  // TEST FOR MONDAY
  transitNetwork.parseFromGTFSFiles(testDirectory, 0, transferTimeInSeconds);

  // remove files and folder...
  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
  remove(stopsFileName.c_str());
  remove(frequenciesFileName.c_str());
  remove(stopTimesFileName.c_str());
  status = rmdir(testDirectory.c_str());
  if (status != 0)
    printf("couldn't remove directory %s\n", testDirectory.c_str());

  Time startTime;
  int stopId;
  vector<int> startNodeIds;

  // 2 Test at midday..
  stopId = transitNetwork.getStopId("Bayerischer Platz");
  startTime = Time(11, 30, 0);
  transitNetwork.getStartNodeIds(stopId, startTime, &startNodeIds);
  ASSERT_EQ(3, startNodeIds.size());
  NodeExpanded* node;
  // check correct stopIds
  for (size_t i = 0; i < startNodeIds.size(); i++)
  {
    node = transitNetwork.getNode(startNodeIds[i]);
    ASSERT_EQ(stopId, node->stopId);
    if (i < startNodeIds.size() - 1)
      ASSERT_EQ(DEPARTURENODE, node->type);
    else  // last node should be a transfer node...
      ASSERT_EQ(TRANSFERNODE, node->type);
  }
  node = transitNetwork.getNode(startNodeIds[0]);
  ASSERT_EQ(11, node->time.hour());
  ASSERT_EQ(33, node->time.minute());
  ASSERT_EQ(0, node->time.second());

  node = transitNetwork.getNode(startNodeIds[1]);
  ASSERT_EQ(11, node->time.hour());
  ASSERT_EQ(33, node->time.minute());
  ASSERT_EQ(6, node->time.second());

  node = transitNetwork.getNode(startNodeIds[2]);
  ASSERT_EQ(11, node->time.hour());
  ASSERT_EQ(36, node->time.minute());
  ASSERT_EQ(0, node->time.second());

  // test at night, needing nodes from next day
  startNodeIds.clear();
  stopId = transitNetwork.getStopId("Berliner Strasse");
  startTime = Time(23, 38, 6);
  transitNetwork.getStartNodeIds(stopId, startTime, &startNodeIds);
  ASSERT_EQ(2, startNodeIds.size());
  // checking stop id
  for (size_t i = 0; i < startNodeIds.size(); i++)
  {
    node = transitNetwork.getNode(startNodeIds[i]);
    ASSERT_EQ(stopId, node->stopId);
  }
  node = transitNetwork.getNode(startNodeIds[0]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(0, node->time.hour());
  ASSERT_EQ(35, node->time.minute());
  ASSERT_EQ(0, node->time.second());

  node = transitNetwork.getNode(startNodeIds[1]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(0, node->time.hour());
  ASSERT_EQ(38, node->time.minute());
  ASSERT_EQ(5, node->time.second());

  // test on night, staying on this day..
  startNodeIds.clear();
  stopId = transitNetwork.getStopId("Eisenacher Strasse");
  startTime = Time(23, 34, 0);
  transitNetwork.getStartNodeIds(stopId, startTime, &startNodeIds);
  ASSERT_EQ(1, startNodeIds.size());

  node = transitNetwork.getNode(startNodeIds[0]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(23, node->time.hour());
  ASSERT_EQ(34, node->time.minute());
  ASSERT_EQ(0, node->time.second());
  // test in morning, having to wait...
  startNodeIds.clear();
  stopId = transitNetwork.getStopId("Guentzelstrasse");
  startTime = Time(2, 38, 14);
  transitNetwork.getStartNodeIds(stopId, startTime, &startNodeIds);
  ASSERT_EQ(2, startNodeIds.size());
  // checking stop id
  for (size_t i = 0; i < startNodeIds.size(); i++)
  {
    node = transitNetwork.getNode(startNodeIds[i]);
    ASSERT_EQ(stopId, node->stopId);
  }
  node = transitNetwork.getNode(startNodeIds[0]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(5, node->time.hour());
  ASSERT_EQ(38, node->time.minute());
  ASSERT_EQ(0, node->time.second());

  node = transitNetwork.getNode(startNodeIds[1]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(5, node->time.hour());
  ASSERT_EQ(38, node->time.minute());
  ASSERT_EQ(13, node->time.second());
}

TEST(ExpandedDijkstraTest, computeShortestPath)
{
  // TODO(Robin): check the length that the routeplanner returns is correct
  TransitNetworkExpanded transitNetwork;
  string testDirectory = "./GTFSTempTest";
  int status = mkdir("./GTFSTempTest", S_IRWXU | S_IRWXG | S_IROTH |
                                            S_IXOTH);
  if (status != 0)
  {
    printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
    // make test crash if it directory could not be created...
    ASSERT_TRUE(false);
  }
  string calendarFileName = testDirectory +"/calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);

  string tripFileName = testDirectory + "/trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n");
  fclose(tripFile);

  string stopsFileName = testDirectory + "/stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
    "location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U9_U7_1,Berliner Strasse, Zu U9 :),52.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);

  string frequenciesFileName = testDirectory + "/frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:00,27:30:00,10800\n"
    "U7WE,00:00:00,24:00:00,5400\n"
    "U7WD,05:30:00,26:30:00,3600\n"
    "U7FW,05:29:32,25:29:32,7200\n");
  fclose(frequenciesFile);

  string stopTimesFileName = testDirectory + "/stop_times.txt";
  FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
  fprintf(stopTimesFile,
    "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
    "pickup_type,dropoff_type\n"
    "U9FW,5:30:0,5:32:17,U9_U7_1,1,0,0,0\n"
    "U9FW,5:34:13,05:38:0,U92,2,0,0,0\n"
    // last stop transfer and departure time equal if transfer time = 4 minutes
    "U7WE,0:0:0,0:1:0,U71,1,0,0,0\n"
    "U7WE,0:4:0,0:6:0,U9_U7_1,2,0,0,0\n"
    "U7WD,5:30:0,05:31:0,U70,1,0,0,0\n"
    "U7WD,5:32:17,05:33:6,U71,2,0,0,0\n"
    "U7WD,5:34:05,05:35:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:29:32,5:30:0,U70,1,0,0,0\n"
    "U7FW,5:32:0,5:33:0,U71,2,0,0,0\n"
    "U7FW,5:34:0,5:35:09,U9_U7_1,3,0,0,0\n"
    "U7FW,5:36:07,5:37:0,U73,4,0,0,0\n");

  fclose(stopTimesFile);
  int transferTimeInSeconds = 4 * 60;
  // TEST FOR MONDAY
  char testDay = 0;
  transitNetwork.parseFromGTFSFiles(testDirectory, testDay,
                                    transferTimeInSeconds);

  // remove files and folder...
  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
  remove(stopsFileName.c_str());
  remove(frequenciesFileName.c_str());
  remove(stopTimesFileName.c_str());
  status = rmdir(testDirectory.c_str());
  if (status != 0)
    printf("couldn't remove directory %s\n", testDirectory.c_str());

  // TEST : Direct Way from Berliner Strasse to Guentzelstrasse ...
  ExpandedDijkstra routePlanner(&transitNetwork);
  routePlanner.computeShortestPath("Berliner Strasse",
                                   Time(5, 29, 31),
                                   "Guentzelstrasse",
                                   true);
  ASSERT_EQ(2, routePlanner.transitShortestPath.size());
  // first element should be starting stop
  NodeExpanded firstNode = routePlanner.transitShortestPath[0].first;
  ASSERT_EQ(transitNetwork.getStopId("Berliner Strasse"),
            firstNode.stopId);
  ASSERT_EQ(5, firstNode.time.hour());
  ASSERT_EQ(32, firstNode.time.minute());
  ASSERT_EQ(17, firstNode.time.second());
  ASSERT_EQ(DEPARTURENODE, firstNode.type);
  // last element should be destination stop..
  NodeExpanded lastNode = routePlanner.transitShortestPath.back().first;
  ASSERT_EQ(transitNetwork.getStopId("Guentzelstrasse"), lastNode.stopId);
  ASSERT_EQ(5, lastNode.time.hour());
  ASSERT_EQ(34, lastNode.time.minute());
  ASSERT_EQ(13, lastNode.time.second());
  ASSERT_EQ(ARRIVALNODE, lastNode.type);


  // now a bit later will lead to 3 hours later (due to so much waiting time...)
  routePlanner.computeShortestPath("Berliner Strasse", Time(5, 38, 0),
                                   "Guentzelstrasse", true);
  // first element should be starting stop
  firstNode = routePlanner.transitShortestPath[0].first;
  ASSERT_EQ(transitNetwork.getStopId("Berliner Strasse"),
            firstNode.stopId);
  ASSERT_EQ(5, firstNode.time.hour());
  ASSERT_EQ(38, firstNode.time.minute());
  ASSERT_EQ(TRANSFERNODE, firstNode.type);
  // second last element should be starting stop...
  NodeExpanded secondLastNode = routePlanner.transitShortestPath[
             (routePlanner.transitShortestPath.size() - 2)].first;
  ASSERT_EQ(transitNetwork.getStopId("Berliner Strasse"),
            secondLastNode.stopId);
  ASSERT_EQ(8, secondLastNode.time.hour());
  ASSERT_EQ(32, secondLastNode.time.minute());
  ASSERT_EQ(17, secondLastNode.time.second());
  ASSERT_EQ(DEPARTURENODE, secondLastNode.type);
  // last element should be destination stop..
  lastNode = routePlanner.transitShortestPath.back().first;
  ASSERT_EQ(transitNetwork.getStopId("Guentzelstrasse"), lastNode.stopId);
  ASSERT_EQ(8, lastNode.time.hour());
  ASSERT_EQ(34, lastNode.time.minute());
  ASSERT_EQ(13, lastNode.time.second());
  ASSERT_EQ(ARRIVALNODE, lastNode.type);

  // A Trip with transfers and waiting time from Bayerischer
  // Platz to Guentzelstrasse

  routePlanner.computeShortestPath("Bayerischer Platz", Time(3, 4, 0),
                                   "Guentzelstrasse", true);
  //ASSERT_EQ(6, routePlanner.transitShortestPath->size());
  // first we run through some transfer nodes
  // before departing to berliner strasse
  size_t nodeNr = 0;
  NodeExpanded routeNode = routePlanner.transitShortestPath[nodeNr].first;
  while (routeNode.type == TRANSFERNODE)
  {
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
    ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
                   routePlanner.transitShortestPath[nodeNr].first.stopId));

  }
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ(33, routeNode.time.minute());
  ASSERT_EQ(6, routeNode.time.second());
  nodeNr++;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ(34, routeNode.time.minute());
  // now some possibly some more transfer nodes...
  nodeNr++;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  while (routeNode.type == TRANSFERNODE)
  {
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(8, routeNode.time.hour());
  ASSERT_EQ(32, routeNode.time.minute());
  ASSERT_EQ(17, routeNode.time.second());
  ASSERT_EQ(116, routePlanner.transitShortestPath[nodeNr].second.time);
  nodeNr++;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(8, routeNode.time.hour());
  ASSERT_EQ(34, routeNode.time.minute());
  ASSERT_EQ(13, routeNode.time.second());

  // checking all ,edges on the trip for correct size... (last edge
  // is a dummy edge, does not need to be checked!)
  for (size_t i = 0; i < routePlanner.transitShortestPath.size() - 1; i++)
  {
    const NodeExpanded& routeNodeFrom = routePlanner.transitShortestPath[i].first;
    const NodeExpanded& routeNodeTo = routePlanner.transitShortestPath[i + 1].first;
    const Edge& routeEdge = routePlanner.transitShortestPath[i].second;
    // edge time should be the time difference inbetween the nodes...
    ASSERT_EQ(routeEdge.time, routeNodeTo.time.
                               minusInSecondsDependent(routeNodeFrom.time));

  }

  // Overnight test: going from bayerischer platz to blissestrasse,
  // crossing the day boundary:

  routePlanner.computeShortestPath("Bayerischer Platz", Time(23, 33, 06),
                                   "Blissestrasse", true);

  // Starting at Bayerischer Platz
  nodeNr = 0;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;

  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ(23, routeNode.time.hour());
  ASSERT_EQ(33, routeNode.time.minute());
  ASSERT_EQ(6, routeNode.time.second());
  nodeNr++;

  // arriving at berliner strasse, waiting for other trip afterwards

  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ(23, routeNode.time.hour());
  ASSERT_EQ(34, routeNode.time.minute());
  ASSERT_EQ(5, routeNode.time.second());

  // now some possibly some transfer nodes...
  nodeNr++;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  while (routeNode.type == TRANSFERNODE)
  {
    ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(1, routeNode.time.hour());
  ASSERT_EQ(35, routeNode.time.minute());
  ASSERT_EQ(9, routeNode.time.second());
  nodeNr++;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ("Blissestrasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(1, routeNode.time.hour());
  ASSERT_EQ(36, routeNode.time.minute());
  ASSERT_EQ(7, routeNode.time.second());

  // checking all ,edges on the trip for correct size... (last edge
  // is a dummy edge, does not need to be checked!)
  for (size_t i = 0; i < routePlanner.transitShortestPath.size() - 1; i++)
  {
    const NodeExpanded& routeNodeFrom = routePlanner.transitShortestPath[i].first;
    const NodeExpanded& routeNodeTo = routePlanner.transitShortestPath[i + 1].first;
    const Edge& routeEdge = routePlanner.transitShortestPath[i].second;
    // edge time should be the time difference inbetween the nodes...
    ASSERT_EQ(routeEdge.time, routeNodeTo.time.
                               minusInSecondsDependent(routeNodeFrom.time));

  }

  // now test a trip that is supposed to start at the end of the day, but where
  // you have to wait until next day...

  routePlanner.computeShortestPath("Berliner Strasse", Time(23, 38, 06),
                                   "Guentzelstrasse", true);
  // Starting at Berliner Strasse, there could be transfer nodes before
  // 2 am
  nodeNr = 0;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  while (routeNode.type == TRANSFERNODE)
  {
    ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
    ASSERT_TRUE(routeNode.time.hour() == 0 || routeNode.time.hour() == 1);
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ(2, routeNode.time.hour());
  ASSERT_EQ(32, routeNode.time.minute());
  ASSERT_EQ(17, routeNode.time.second());
  nodeNr++;

  // arriving at guenztelstrasse...
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ(2, routeNode.time.hour());
  ASSERT_EQ(34, routeNode.time.minute());
  ASSERT_EQ(13, routeNode.time.second());

  // checking all ,edges on the trip for correct size... (last edge
  // is a dummy edge, does not need to be checked!)
  for (size_t i = 0; i < routePlanner.transitShortestPath.size() - 1; i++)
  {
    const NodeExpanded& routeNodeFrom = routePlanner.transitShortestPath[i].first;
    const NodeExpanded& routeNodeTo = routePlanner.transitShortestPath[i + 1].first;
    const Edge& routeEdge = routePlanner.transitShortestPath[i].second;
    // edge time should be the time difference inbetween the nodes...
    ASSERT_EQ(routeEdge.time, routeNodeTo.time.
                               minusInSecondsDependent(routeNodeFrom.time));
  }

  // Overnight test with transfer: going from eisenacher strasse to
  // guentzelstrasse, crossing the day boundary:

  routePlanner.computeShortestPath("Eisenacher Strasse", Time(23, 30, 00),
                                   "Guentzelstrasse", true);

  // Starting at Eisenacher Strasse
  nodeNr = 0;
  routeNode = routePlanner.transitShortestPath[nodeNr].first;

  ASSERT_EQ("Eisenacher Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(23, routeNode.time.hour());

  // now possibly waiting at eisenacher strasse...
  while (transitNetwork.getStopName(routeNode.stopId) == "Eisenacher Strasse")
  {
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }
  // arriving at Bayierhsc platz, possibly waiting,
  // then contiuing to berliner strasse
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  while (transitNetwork.getStopName(routeNode.stopId) == "Bayerischer Platz")
  {
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }

  // arriving at berliner strasse, waiting for other trip afterwards

  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  while (transitNetwork.getStopName(routeNode.stopId) == "Berliner Strasse")
  {
    nodeNr++;
    routeNode = routePlanner.transitShortestPath[nodeNr].first;
  }
  nodeNr--;  // for also checking departure node from Berliner Strasse

  // now check last two nodes, should be departure from berliner to arrival
  // at guentzelstrasse
  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ(DEPARTURENODE, routeNode.type);
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(2, routeNode.time.hour());
  ASSERT_EQ(32, routeNode.time.minute());
  ASSERT_EQ(17, routeNode.time.second());
  nodeNr++;

  routeNode = routePlanner.transitShortestPath[nodeNr].first;
  ASSERT_EQ(ARRIVALNODE, routeNode.type);
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(routeNode.stopId));
  ASSERT_EQ(2, routeNode.time.hour());
  ASSERT_EQ(34, routeNode.time.minute());
  ASSERT_EQ(13, routeNode.time.second());

  // check size of path
  ASSERT_EQ(nodeNr + 1, routePlanner.transitShortestPath.size());
  // checking all ,edges on the trip for correct size... (last edge
  // is a dummy edge, does not need to be checked!)
  for (size_t i = 0; i < routePlanner.transitShortestPath.size() - 1; i++)
  {
    const NodeExpanded& routeNodeFrom = routePlanner.transitShortestPath[i].first;
    const NodeExpanded& routeNodeTo = routePlanner.transitShortestPath[i + 1].first;
    const Edge& routeEdge = routePlanner.transitShortestPath[i].second;
    // edge time should be the time difference inbetween the nodes...
    ASSERT_EQ(routeEdge.time, routeNodeTo.time.
                               minusInSecondsDependent(routeNodeFrom.time));
  }
}
