// Copyright November 22, 2011 Robin Schirrmeister and Simon Skilevic
// Chair of Algorithms and Datastructures Uni Freiburg



#include "HybridTransitRoadNetwork.h"
using std::vector;
// _____________________________________________________________________________
HybridTransitRoadNetwork::HybridTransitRoadNetwork(RoadNetwork* roadNetwork,
                              TransitNetworkDependent* transitNetworkDependent,
                                TransitNetworkExpanded* transitNetworkExpanded)
{
  // some sanity checks: there should be atleast one network, there should
  // be only one transit Network for now..
  assert(roadNetwork != NULL || transitNetworkDependent != NULL
         || transitNetworkExpanded != NULL);
  assert(transitNetworkExpanded == NULL || transitNetworkDependent == NULL);
  _roadNetwork = roadNetwork;
  _transitNetworkDependent = transitNetworkDependent;
  _transitNetworkExpanded = transitNetworkExpanded;
}
// _____________________________________________________________________________
HybridTransitRoadNetwork::~HybridTransitRoadNetwork()
{
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::addNaighbourConnections(int nodeId,
                                         NetworkType type,
                                         int endId,
                                         NetworkType endType,
                                         bool startStation,
                                         Time arrivalTime,
                                         vector<LiveConnection>* outConnections)
{
  // printf("adding neighbours....\n");
  if (_roadIdToTransitIds.size() == 0)
    return;
  if (type == ROADNETWORK)
  {
    for (size_t i = 0; i < _roadIdToTransitIds[nodeId].size(); i++)
    {
       int newNodeId = _roadIdToTransitIds[nodeId][i].otherNodeId;
       int newTime = _roadIdToTransitIds[nodeId][i].time;
       assert(newTime >= 0);
       assert(newNodeId>=0);
       Connection connection(arrivalTime, arrivalTime, -1);
       connection.arrivalTime.addSeconds(newTime);
       if (isExpanded())
       {
         vector<int> expandedNodeIds;
         Time buf = connection.arrivalTime;
         buf.addSeconds(4 * 60);
         _transitNetworkExpanded->getStartNodeIds(newNodeId,
                                                 buf, &expandedNodeIds);
         for (size_t j = 0; j < expandedNodeIds.size(); j++)
         {
           Time newNodeTime = _transitNetworkExpanded->
                                  getNode(expandedNodeIds[j])->time;
           int timeFromArrToDep = 0;
           Connection newConnection = connection;
           bool endStation = equalWithEndStation(endType, endId, TRANSITNETWORK, expandedNodeIds[j]);
           if (!endStation)
           {
             timeFromArrToDep = newNodeTime.minusInSecondsDependent(
                                                      connection.arrivalTime) + 4*60;
             newConnection.arrivalTime = newNodeTime;
           }
           // HAAAAAACKKK (SIMON) BEGIN
           Time testTime = connection.arrivalTime;
           testTime.addSeconds(timeFromArrToDep);
           if (testTime < newConnection.arrivalTime && !endStation)
           {
             timeFromArrToDep += newConnection.arrivalTime.minusInSecondsDependent(testTime);
           }
           // HAAAAAACKKK (SIMON) END
           size_t transfer = 0;//newTime + timeFromArrToDep;
//           if (!endStation && !startStation &&_transitNetworkExpanded->getNode(expandedNodeIds[j])->type
//                  == DEPARTURENODE)
           if (!endStation && !startStation)
             transfer+=1;
           newConnection.arrivalTime.addSeconds(timeFromArrToDep);
           LiveConnection liveConnection(newConnection, TRANSITNETWORK,
                                         expandedNodeIds[j],
                                         newTime + timeFromArrToDep,
                                         transfer, false);
           outConnections->push_back(liveConnection);
         }
       }
       else
       {
         bool endStation = equalWithEndStation(endType, endId, TRANSITNETWORK, newNodeId);
         size_t transfer = 0;
         if (!endStation && !startStation)
           transfer++;
         LiveConnection liveConnection(connection, TRANSITNETWORK,
                                       newNodeId, newTime,
                                       transfer, false);
         outConnections->push_back(liveConnection);
       }
    }
  }
  else
  {
    if (isExpanded())
      nodeId = _transitNetworkExpanded->getNode(nodeId)->stopId;
    for (size_t i = 0; i < _transitIdToRoadIds[nodeId].size(); i++)
    {
      int newNodeId = _transitIdToRoadIds[nodeId][i].otherNodeId;
      int newTime = _transitIdToRoadIds[nodeId][i].time;
      bool endStation = equalWithEndStation(endType, endId, ROADNETWORK, newNodeId);
      size_t transfer = 0;
//      if (!startStation)
//        transfer = 10;
     if (!endStation && !startStation)
        transfer++;
      assert(newTime > 0 && newNodeId >= 0);
      Connection connection(arrivalTime, arrivalTime, -1);
      connection.arrivalTime.addSeconds(newTime);
      LiveConnection liveConnection(connection, ROADNETWORK,
                                    newNodeId, newTime,
                                    transfer, false);
      outConnections->push_back(liveConnection);
    }
  }
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::getStartConnections(int startId,
                                       NetworkType startNodeType,
                                       const Time& startTime,
                                       vector<LiveConnection>* startConnections)
{
  assert(startConnections->size() == 0);
  if (startNodeType == ROADNETWORK)
  {
    int dummyTripId = -1;
    Connection startConnection(startTime, startTime, dummyTripId);
    LiveConnection startLiveConnection(startConnection, ROADNETWORK,
                                       startId, 0,
                                       0, false);
    startConnections->push_back(startLiveConnection);
  }
  else if (startNodeType == TRANSITNETWORK && _transitNetworkDependent != NULL)
  {
    assert(_transitNetworkExpanded == NULL);
    int dummyTripId = -1;
    Connection startConnection(startTime, startTime, dummyTripId);
    bool tripBeforeEnded = false;
    int time = 0;
    size_t transfers  = 0;
    LiveConnection startLiveConnection(startConnection, TRANSITNETWORK,
                                       startId, time,
                                       transfers, tripBeforeEnded);
    startConnections->push_back(startLiveConnection);
  }
  else if (startNodeType == TRANSITNETWORK && _transitNetworkExpanded != NULL)
  {
    int stopId = startId;
    vector<int> startNodeIds;
    _transitNetworkExpanded->getStartNodeIds(stopId, startTime, &startNodeIds);
    if (startNodeIds.size() == 0)
    {
      printf("No start nodes at id %d\n", stopId);
      return;
    }

    for (size_t i = 0; i < startNodeIds.size(); i++)
    {
      Time nodeTime = _transitNetworkExpanded->getNode(startNodeIds[i])->time;
      int currentTimeFromStart = nodeTime.minusInSecondsDependent(startTime);
      size_t transfers  = 0;
      int tripId = _transitNetworkExpanded->getTripId(startNodeIds[i]);
      Connection startConnection(startTime, startTime, tripId);
      startConnection.arrivalTime.addSeconds(currentTimeFromStart);
      LiveConnection newConnection(startConnection, TRANSITNETWORK,
                                  startNodeIds[i],
                                  currentTimeFromStart,
                                  transfers, false);
      startConnections->push_back(newConnection);
    }
    Connection startConnection(startTime, startTime, -1);
    LiveConnection dummyConnection(startConnection, TRANSITNETWORK,                                 
                                  _transitNetworkExpanded->getNumNodes() - 1,
                                  0,
                                  0, false);
    startConnections->push_back(dummyConnection);
//    addNaighbourConnections(_transitNetworkExpanded->getNumNodes() - 1,
//                            TRANSITNETWORK, -1,
//                            TRANSITNETWORK, startTime, startConnections);
  }
  else
  {
    // should never come here
    printf("no suitable network type found in get start nodes\n");
    assert(false);
  }
}
// _____________________________________________________________________________
int HybridTransitRoadNetwork::getStopIdByTheStopName(string stopName)
{
  if (isExpanded())
    return _transitNetworkExpanded->getStopId(stopName);
  else
    return _transitNetworkDependent->getNodeIdByTheStopName(stopName);
}
// _____________________________________________________________________________
bool HybridTransitRoadNetwork::isExpanded()
{
  return _transitNetworkExpanded != NULL;
}
// _____________________________________________________________________________
int HybridTransitRoadNetwork::getLocationId(NetworkType type, int nodeId)
{
  if (isExpanded() && type == TRANSITNETWORK)
    return _transitNetworkExpanded->getNode(nodeId)->stopId;
  else
    return nodeId;
}
// _____________________________________________________________________________
NodeExpanded* HybridTransitRoadNetwork::getNode(NetworkType type, int nodeId)
{
  if (type == TRANSITNETWORK)
  {
    assert(_transitNetworkExpanded != NULL);
    return _transitNetworkExpanded->getNode(nodeId);
  }
  else if (type == ROADNETWORK)
  {
    assert(_roadNetwork != NULL);
    return _roadNetwork->getNode(nodeId);
  }
  else
  {
    printf("no suitable network type found\n");
    assert(false);
  }
}
// _____________________________________________________________________________
int HybridTransitRoadNetwork::getNumNodes(NetworkType type)
{
  if (type == TRANSITNETWORK)
  {
    if (isExpanded())
      return _transitNetworkExpanded->getNumNodes();
    else
      return _transitNetworkDependent->getNumNodes();
  } else if (type == ROADNETWORK)
  {
    if (_roadNetwork != NULL)
      return _roadNetwork->getNumNodes();
    else
      return 0;
  }
  else
  {
    printf("Size of unknown Network Type requested\n");
    assert(false);
  }
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::getConnections(int nodeId, NetworkType type,
                                         int endId,
                                         NetworkType endType,
                                         Time arrivalTime,
                                         int tripId, bool isMultiCriteria,
                                         bool isStartNode,
                                         vector<LiveConnection>* outConnections)
{
   if (type == TRANSITNETWORK && !isExpanded())
   {

      _transitNetworkDependent->getConnections(nodeId, arrivalTime, tripId,
                                                   isMultiCriteria, isStartNode,
                                                   outConnections);
      addNaighbourConnections(nodeId, type, endId, endType, isStartNode, arrivalTime, outConnections);
   }
   else if (type == TRANSITNETWORK && isExpanded())
   {
     if (nodeId != _transitNetworkExpanded->getNumNodes() - 1)
      _transitNetworkExpanded->getConnections(nodeId, isStartNode, arrivalTime,
                                                   outConnections);
      if (_transitNetworkExpanded->getNode(nodeId)->type == ARRIVALNODE)
        addNaighbourConnections(nodeId, type, endId, endType, isStartNode, arrivalTime, outConnections);
   }
   else if (type == ROADNETWORK)
   {
      _roadNetwork->getConnections(nodeId, arrivalTime, isStartNode,
                                                   outConnections);
      addNaighbourConnections(nodeId, type, endId, endType, isStartNode, arrivalTime, outConnections);
   }

}
// _____________________________________________________________________________
bool HybridTransitRoadNetwork::equalWithStartStation(NetworkType nodeType,
                                                int startId,
                                                NetworkType otherNodeType,
                                                int otherNodeId)
{
  if (nodeType != otherNodeType)
    return false;
  else if (nodeType == ROADNETWORK || !isExpanded())
    return startId == otherNodeId;
  else
    return startId == _transitNetworkExpanded->getNode(otherNodeId)->stopId;
}
// _____________________________________________________________________________
bool HybridTransitRoadNetwork::equalWithEndStation(NetworkType nodeType,
                                                int endId,
                                                NetworkType otherNodeType,
                                                int otherNodeId)
{
  if (nodeType != otherNodeType)
    return false;
  else if (nodeType == ROADNETWORK || !isExpanded())
    return endId == otherNodeId;
  else
    return endId == _transitNetworkExpanded->getNode(otherNodeId)->stopId;
}
// _____________________________________________________________________________
double HybridTransitRoadNetwork::getLatitude(NetworkType type, int nodeId)
{
  if (type == TRANSITNETWORK)
  {
    if (!isExpanded())
      return _transitNetworkDependent->getNode(nodeId)->latitude;
    else
      return _transitNetworkExpanded->getNode(nodeId)->latitude;
  }
  else if (type == ROADNETWORK)
  {
    return _roadNetwork->getNode(nodeId)->latitude;
  }
  else
  {
    printf("no suitable network type found.. type: %d\n", type);
    assert(false);
  }
}
// _____________________________________________________________________________
double HybridTransitRoadNetwork::getLongitude(NetworkType type, int nodeId)
{
  if (type == TRANSITNETWORK)
  {
    if (!isExpanded())
      return _transitNetworkDependent->getNode(nodeId)->longitude;
    else
      return _transitNetworkExpanded->getNode(nodeId)->longitude;
  }
  else if (type == ROADNETWORK)
  {
    return _roadNetwork->getNode(nodeId)->longitude;
  }
  else
  {
    printf("no suitable type found..");
    assert(false);
  }
}
// _____________________________________________________________________________
int HybridTransitRoadNetwork::getTripId(NetworkType netType, int nodeId)
{
  if (netType == TRANSITNETWORK)
  {
    assert(_transitNetworkExpanded != NULL);
    return _transitNetworkExpanded->getTripId(nodeId);
  }
  else if (netType == ROADNETWORK)
  {
    assert(_roadNetwork != NULL);
    return -1;
  }
  else
  {
    printf("no suitable network type found\n");
    assert(false);
  }
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::buildHybridNetwork(int naighboursNumber)
{
  assert(_roadNetwork != NULL && (_transitNetworkDependent != NULL
          || _transitNetworkExpanded != NULL));
  _roadIdToTransitIds.resize(_roadNetwork->getNumNodes());
  if (isExpanded())
    _transitIdToRoadIds.resize(_transitNetworkExpanded->getNumStops());
  else
    _transitIdToRoadIds.resize(_transitNetworkDependent->getNumNodes());

  for (size_t i = 0; i < _transitIdToRoadIds.size(); i++)
  {
    double lat = 0;
    double lon = 0;
    if (isExpanded())
    {
      lat = _transitNetworkExpanded->getStop(i)->latitude;
      lon = _transitNetworkExpanded->getStop(i)->longitude;
    } else
    {
      lat = _transitNetworkDependent->getNode(i)->latitude;
      lon = _transitNetworkDependent->getNode(i)->longitude;
    }
    int speedInKmh = 5;
    _roadNetwork->getNearestNodesIdsSIMPLE(lat, lon, naighboursNumber,
            speedInKmh,
            &(_transitIdToRoadIds[i]));
    assert(_transitIdToRoadIds[i].size() == naighboursNumber
           || _transitIdToRoadIds[i].size() == _roadNetwork->getNumNodes());
    for (size_t j = 0; j < _transitIdToRoadIds[i].size(); j++)
      _roadIdToTransitIds[_transitIdToRoadIds[i][j].otherNodeId]
                          .push_back(Edge(i, _transitIdToRoadIds[i][j].time));
  }
  if (isExpanded())
    assert(_transitIdToRoadIds.size() ==
           _transitNetworkExpanded->getNumStops());
  else
    assert(_transitIdToRoadIds.size() ==
           _transitNetworkDependent->getNumStops());
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::getStopsStartingWith(const string& prefix,
                                                    vector<string>* stopNames)
{
  if (!isExpanded())
  return _transitNetworkDependent->getStopsStartingWith(prefix, stopNames);
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::addDummyNode(int startId, const Time& startTime)
{
  NodeExpanded node(_transitNetworkExpanded->getNumNodes(),
                    _transitNetworkExpanded->getStop(startId)->latitude,
                    _transitNetworkExpanded->getStop(startId)->longitude,
                    startId, startTime, ARRIVALNODE);
  _transitNetworkExpanded->addDummyNode(node);
}
// _____________________________________________________________________________
void HybridTransitRoadNetwork::removeDummyNode()
{
  _transitNetworkExpanded->removeDummyNode();
}
// _____________________________________________________________________________
int HybridTransitRoadNetwork::getNumStops(NetworkType type)
{
  if (type == ROADNETWORK)
    return _roadNetwork->getNumNodes();
  else
  {
    if (isExpanded())
      return _transitNetworkExpanded->getNumStops();
    else
      return _transitNetworkDependent->getNumNodes();
  }
}
// _____________________________________________________________________________