/* Copyright November 8, 2011 Robin Schirrmeister and Simon Skilevic
 * Chair of Algorithms and Datastructures Uni Freiburg
 * Uni Freiburg
 * Testclass for Transit network
 */

#include <stdio.h>
#include <sys/stat.h>
#include "./CsvParser.h"
#include "./TransitNetworkExpanded.h"
#include "./Dijkstra.h"
#include "./ExpandedDijkstra.h"
#include "./MulticriteriaDependentDijkstra.h"
#include "MulticriteriaExpandedDijkstra.h"

using std::string;
using std::unordered_map;

TEST(MulticriteriaDependentTest, computeShortestPath)
{
  {
   TransitNetworkDependent transitNetwork;

  string testDirectory = "./GTFSMultiDependentTest";
  int status = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG | S_IROTH |
                                            S_IXOTH);
  if (status != 0)
  {
    printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
    // make test crash if it directory could not be created...
    ASSERT_TRUE(false);
  }
  string calendarFileName =  testDirectory + "/calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);

  string tripFileName =  testDirectory + "/trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n"
                        "A,FW,SHORTCUTTRANSFER,Downtown,2\n"
                        "A,WD,SHORTCUTTIME,Downtown,2\n");  // add extra trips
                                                     // for transfer tests...
  fclose(tripFile);

  string stopsFileName = testDirectory + "/stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
    "location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U9_U7_1,Berliner Strasse, Zu U9 :),52.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);

  string frequenciesFileName = testDirectory + "/frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:00,25:30:00,10800\n"
    "U7WE,00:00:00,24:00:00,5400\n"
    "U7WD,05:26:00,26:30:00,3600\n"
    "U7FW,05:29:32,25:00:00,7200\n"
    "SHORTCUTTRANSFER,05:29:32,25:00:00,7200\n"
    "SHORTCUTTIME,05:29:32,25:00:00,7200\n");
  fclose(frequenciesFile);

  string stopTimesFileName = testDirectory + "/stop_times.txt";
  FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
  fprintf(stopTimesFile,
    "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
    "pickup_type,dropoff_type\n"
    "U9FW,5:30:0,5:32:17,U9_U7_1,1,0,0,0\n"
    "U9FW,5:34:13,05:38:0,U92,2,0,0,0\n"
    // last stop transfer and departure time equal if transfer time = 4 minutes
    "U7WE,0:0:0,0:1:0,U71,1,0,0,0\n"
    "U7WE,0:4:0,0:6:0,U9_U7_1,2,0,0,0\n"
    "U7WD,5:26:0,05:26:32,U70,1,0,0,0\n"
    "U7WD,5:26:58,05:27:10,U71,2,0,0,0\n"
    "U7WD,5:27:35,05:29:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:29:32,5:30:0,U70,1,0,0,0\n"
    "U7FW,5:32:0,5:33:0,U71,2,0,0,0\n"
    "U7FW,5:34:0,5:35:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:36:0,5:37:0,U73,4,0,0,0\n"
    "SHORTCUTTRANSFER,5:29:32,5:30:0,U71,1,0,0,0\n"
    "SHORTCUTTRANSFER,5:38:0,5:40:0,U92,2,0,0,0\n"
    "SHORTCUTTIME,5:29:32,5:30:58,U71,1,0,0,0\n"
    "SHORTCUTTIME,5:32:02,5:32:0,U73,2,0,0,0\n");
  fclose(stopTimesFile);

  transitNetwork.parseFromGTFSFiles(testDirectory);

  TransitNetworkExpanded transitNetworkExpanded;
  transitNetworkExpanded.setVerbosity(2);
  transitNetworkExpanded.parseFromGTFSFiles(testDirectory, 0, 4 * 60);
  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
  remove(stopsFileName.c_str());
  remove(frequenciesFileName.c_str());
  remove(stopTimesFileName.c_str());
  status = rmdir(testDirectory.c_str());
  if (status != 0)
    printf("couldn't remove directory %s\n", testDirectory.c_str());
  // TEST : Direct Way from Berliner Strasse to Guentzelstrasse ...
  MulticriteriaDependentDijkstra routePlanner(&transitNetwork);
  routePlanner.computeShortestPath("Berliner Strasse",
                                   Time(5, 29, 31),
                                   "Guentzelstrasse",
                                   true);

  // trip should just go at 5:32:17 from berliner strasse to 5:34:13 at
  // guentzelstrasse
  ASSERT_EQ(1, routePlanner.optimalPaths.size());
  ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(0)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getDepartureTime(0).hour());
  ASSERT_EQ(32, routePlanner.optimalPaths[0].getDepartureTime(0).minute());
  ASSERT_EQ(17, routePlanner.optimalPaths[0].getDepartureTime(0).second());
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(1)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getArrivalTime(1).hour());
  ASSERT_EQ(34, routePlanner.optimalPaths[0].getArrivalTime(1).minute());
  ASSERT_EQ(13, routePlanner.optimalPaths[0].getArrivalTime(1).second());

  // make a trip where 2 trips are available but only one should be found
    routePlanner.computeShortestPath("Eisenacher Strasse",
                                   Time(7, 25, 31),
                                   "Bayerischer Platz",
                                   true);
  ASSERT_EQ(1, routePlanner.optimalPaths.size());
  ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
  ASSERT_EQ("Eisenacher Strasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(0)));
  ASSERT_EQ(7, routePlanner.optimalPaths[0].getDepartureTime(0).hour());
  ASSERT_EQ(26, routePlanner.optimalPaths[0].getDepartureTime(0).minute());
  ASSERT_EQ(32, routePlanner.optimalPaths[0].getDepartureTime(0).second());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(1)));
  ASSERT_EQ(7, routePlanner.optimalPaths[0].getArrivalTime(1).hour());
  ASSERT_EQ(26, routePlanner.optimalPaths[0].getArrivalTime(1).minute());
  ASSERT_EQ(58, routePlanner.optimalPaths[0].getArrivalTime(1).second());

  // Same Test, but should result in the next trip...
  // This test is ehre because it led to an error before...

  // make a trip where 2 trips are available but only one should be found
    routePlanner.computeShortestPath("Eisenacher Strasse",
                                   Time(7, 29, 31),
                                   "Bayerischer Platz",
                                   true);
  ASSERT_EQ(1, routePlanner.optimalPaths.size());
  ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
  ASSERT_EQ("Eisenacher Strasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(0)));
  ASSERT_EQ(7, routePlanner.optimalPaths[0].getDepartureTime(0).hour());
  ASSERT_EQ(30, routePlanner.optimalPaths[0].getDepartureTime(0).minute());
  ASSERT_EQ(0, routePlanner.optimalPaths[0].getDepartureTime(0).second());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(1)));
  ASSERT_EQ(7, routePlanner.optimalPaths[0].getArrivalTime(1).hour());
  ASSERT_EQ(32, routePlanner.optimalPaths[0].getArrivalTime(1).minute());
  ASSERT_EQ(0, routePlanner.optimalPaths[0].getArrivalTime(1).second());

  // MAKE TRIP ON MONDAY 5:27 from bayerischer platz to guentzelstrasse,
  // expect two optimal labels:
  // one from EXTRA Trip, going from bayerischer platz at 5:30:00
  // to guentzelstrasse at 5:38:00
  // other trip from bayerischer platz at 5:27:10 to berliner strasse at 5:27:35
  // then transfer, then from berliner strasse at 5:32:17
  // to guentzelstrasse at 5:34:13
  routePlanner.computeShortestPath("Bayerischer Platz",
                                   Time(5, 27, 10),
                                   "Guentzelstrasse",
                                   true);

  ASSERT_EQ(2, routePlanner.optimalPaths.size());
  // WARNING: Test assumes a certain order of the found trips,
  // altough this order could change
  ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(0)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getDepartureTime(0).hour());
  ASSERT_EQ(30, routePlanner.optimalPaths[0].getDepartureTime(0).minute());
  ASSERT_EQ(0, routePlanner.optimalPaths[0].getDepartureTime(0).second());
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(1)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getArrivalTime(1).hour());
  ASSERT_EQ(38, routePlanner.optimalPaths[0].getArrivalTime(1).minute());
  ASSERT_EQ(0, routePlanner.optimalPaths[0].getArrivalTime(1).second());


  ASSERT_EQ(3, routePlanner.optimalPaths[1].size());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          routePlanner.optimalPaths[1].getStationId(0)));
  ASSERT_EQ(5, routePlanner.optimalPaths[1].getDepartureTime(0).hour());
  ASSERT_EQ(27, routePlanner.optimalPaths[1].getDepartureTime(0).minute());
  ASSERT_EQ(10, routePlanner.optimalPaths[1].getDepartureTime(0).second());
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[1].getStationId(1)));
  ASSERT_EQ(5, routePlanner.optimalPaths[1].getArrivalTime(1).hour());
  ASSERT_EQ(27, routePlanner.optimalPaths[1].getArrivalTime(1).minute());
  ASSERT_EQ(35, routePlanner.optimalPaths[1].getArrivalTime(1).second());
  ASSERT_EQ(5, routePlanner.optimalPaths[1].getDepartureTime(1).hour());
  ASSERT_EQ(32, routePlanner.optimalPaths[1].getDepartureTime(1).minute());
  ASSERT_EQ(17, routePlanner.optimalPaths[1].getDepartureTime(1).second());
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[1].getStationId(2)));
  ASSERT_EQ(5, routePlanner.optimalPaths[1].getArrivalTime(2).hour());
  ASSERT_EQ(34, routePlanner.optimalPaths[1].getArrivalTime(2).minute());
  ASSERT_EQ(13, routePlanner.optimalPaths[1].getArrivalTime(2).second());

  // Similar Trip Test for fast transfer slow no transfer
  routePlanner.computeShortestPath("Eisenacher Strasse",
                                   Time(5, 26, 32),
                                   "Blissestrasse",
                                   true);

  ASSERT_EQ(2, routePlanner.optimalPaths.size());
  // WARNING: Test assumes a certain order of the result routes,
  // although this order could change!
  TransitPath& firstPath = routePlanner.optimalPaths[1];
  ASSERT_EQ(3, firstPath.size());
  ASSERT_EQ("Eisenacher Strasse", transitNetwork.getStopName(
          firstPath.getStationId(0)));
  ASSERT_EQ(5, firstPath.getDepartureTime(0).hour());
  ASSERT_EQ(26, firstPath.getDepartureTime(0).minute());
  ASSERT_EQ(32, firstPath.getDepartureTime(0).second());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          firstPath.getStationId(1)));
  ASSERT_EQ(5, firstPath.getArrivalTime(1).hour());
  ASSERT_EQ(26, firstPath.getArrivalTime(1).minute());
  ASSERT_EQ(58, firstPath.getArrivalTime(1).second());
  ASSERT_EQ(5, firstPath.getDepartureTime(1).hour());
  ASSERT_EQ(30, firstPath.getDepartureTime(1).minute());
  ASSERT_EQ(58, firstPath.getDepartureTime(1).second());
  ASSERT_EQ("Blissestrasse", transitNetwork.getStopName(
          firstPath.getStationId(2)));
  ASSERT_EQ(5, firstPath.getArrivalTime(2).hour());
  ASSERT_EQ(32, firstPath.getArrivalTime(2).minute());
  ASSERT_EQ(2, firstPath.getArrivalTime(2).second());

  TransitPath& secondPath = routePlanner.optimalPaths[0];
  ASSERT_EQ(4, secondPath.size());
  ASSERT_EQ("Eisenacher Strasse", transitNetwork.getStopName(
          secondPath.getStationId(0)));
  ASSERT_EQ(5, secondPath.getDepartureTime(0).hour());
  ASSERT_EQ(30, secondPath.getDepartureTime(0).minute());
  ASSERT_EQ(0, secondPath.getDepartureTime(0).second());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
          secondPath.getStationId(1)));
  ASSERT_EQ(5, secondPath.getArrivalTime(1).hour());
  ASSERT_EQ(32, secondPath.getArrivalTime(1).minute());
  ASSERT_EQ(0, secondPath.getArrivalTime(1).second());
  ASSERT_EQ(5, secondPath.getDepartureTime(1).hour());
  ASSERT_EQ(33, secondPath.getDepartureTime(1).minute());
  ASSERT_EQ(0, secondPath.getDepartureTime(1).second());
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(
          secondPath.getStationId(2)));
  ASSERT_EQ(5, secondPath.getArrivalTime(2).hour());
  ASSERT_EQ(34, secondPath.getArrivalTime(2).minute());
  ASSERT_EQ(0, secondPath.getArrivalTime(2).second());
  ASSERT_EQ(5, secondPath.getDepartureTime(2).hour());
  ASSERT_EQ(35, secondPath.getDepartureTime(2).minute());
  ASSERT_EQ(0, secondPath.getDepartureTime(2).second());
  ASSERT_EQ("Blissestrasse", transitNetwork.getStopName(
          secondPath.getStationId(3)));
  ASSERT_EQ(5, secondPath.getArrivalTime(3).hour());
  ASSERT_EQ(36, secondPath.getArrivalTime(3).minute());
  ASSERT_EQ(0, secondPath.getArrivalTime(3).second());
  }
  // Second Test Set: Trying to construct some Tests based on errors shown
  // during development (i.e. for an error that was found
  // construct a test that should fail if the rror is not fixed)
  {
    // Test: parallel trips with one leaving out stations and continuing until
    // target, the other one visiting all stations but not contiuing until
    // target station. There is an extra transfer trip starting at the
    // last station of the second trip
     TransitNetworkDependent transitNetwork;
     TransitNetworkExpanded tranNetworkExpanded;
    string testDirectory = "./GTFSMultiDependentTestErrors";
    int status = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG | S_IROTH |
                                              S_IXOTH);
    if (status != 0)
    {
      printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
      // make test crash if it directory could not be created...
      ASSERT_TRUE(false);
    }
    string calendarFileName =  testDirectory + "/calendar.txt";
    FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
    fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                          "friday,saturday,sunday,start_date,end_date\n"
                          "FW,1,1,1,1,1,1,1,20060701,20060731\n");
    fclose(calendarFile);

    string tripFileName =  testDirectory + "/trips.txt";
    FILE* tripFile = fopen(tripFileName.c_str(), "w");
    fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                          "A,FW,SLOWTRIP,Downtown,2\n"
                          "A,FW,PARALLELTRIP,Downtown,2\n"
                          "A,FW,TRANSFERTRIP,Downtown,2\n");

    fclose(tripFile);

    string stopsFileName = testDirectory + "/stops.txt";
    FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
    fprintf(stopsFile,
      "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
      "location_type,parent_stop\n"
      "SLOWTRIP0,Startstation,Pizza :),37.728631,-122.431282,,\n"
      "PARALLEL1_SLOW1,Parallel1 / Slow 1,Pizza :),37.728631,-122.431282,,\n"
      "SLOWTRIP2,Slowtrip2, Zu U9 :),52.74103,-122.422482,,\n"
      "PARALLEL2_SLOW3,Parallel2 / Slow 3,Mein Zuhause :),37.728631,-122.433282,,\n"
      "PARALLEL3_SLOW4,Parallel3 / Slow 4,was will man da?,37.75223,-122.418581,,\n"
      "TARGET,Targetstation,hmmm,37.761829,-122.419382,,\n");
    fclose(stopsFile);

    string stopTimesFileName = testDirectory + "/stop_times.txt";
    FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
    fprintf(stopTimesFile,
      "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
      "pickup_type,dropoff_type\n"
      "SLOWTRIP,21:58:22,21:58:22,SLOWTRIP0,1,0,0,0\n"
      "SLOWTRIP,22:0:0,22:0:0,PARALLEL1_SLOW1,2,0,0,0\n"
      "SLOWTRIP,22:0:48,22:0:48,SLOWTRIP2,3,0,0,0\n"
      "SLOWTRIP,22:01:31,22:1:31,PARALLEL2_SLOW3,4,0,0,0\n"
      "SLOWTRIP,22:3:46,22:3:46,PARALLEL3_SLOW4,5,0,0,0\n"
      "TRANSFERTRIP,22:23:0,22:23:0,PARALLEL3_SLOW4,1,0,0,0\n"
      "TRANSFERTRIP,22:25:0,22:25:0,TARGET,2,0,0,0\n"
      "PARALLELTRIP,22:5:0,22:5:0,PARALLEL1_SLOW1,1,0,0,0\n"
      "PARALLELTRIP,22:6:37,22:6:37,PARALLEL2_SLOW3,2,0,0,0\n"
      "PARALLELTRIP,22:9:0,22:9:0,PARALLEL3_SLOW4,3,0,0,0\n"
      "PARALLELTRIP,22:11:0,22:11:0,TARGET,4,0,0,0\n");
    fclose(stopTimesFile);

    transitNetwork.parseFromGTFSFiles(testDirectory);
  tranNetworkExpanded.parseFromGTFSFiles(testDirectory, 0, 4 * 60);
    remove(calendarFileName.c_str());
    remove(tripFileName.c_str());
    remove(stopsFileName.c_str());
    remove(stopTimesFileName.c_str());
    status = rmdir(testDirectory.c_str());
    if (status != 0)
      printf("couldn't remove directory %s\n", testDirectory.c_str());
    MulticriteriaDependentDijkstra routePlanner(&transitNetwork);
    printf("test to fail...\n");
    routePlanner.computeShortestPath("Startstation",
                                     Time(5, 27, 10),
                                     "Targetstation",
                                     true);

    printf(" end\n");
    ASSERT_EQ(1, routePlanner.optimalPaths.size());
    //ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
    ASSERT_EQ("Startstation", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(0)));
    ASSERT_EQ(Time(21, 58, 22), routePlanner.optimalPaths[0].getDepartureTime(0));
    ASSERT_EQ("Parallel1 / Slow 1", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(1)));
    ASSERT_EQ(Time(22, 0, 0), routePlanner.optimalPaths[0].getArrivalTime(1));
    ASSERT_EQ(Time(22, 0, 0), routePlanner.optimalPaths[0].getDepartureTime(1));
    ASSERT_EQ("Slowtrip2", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(2)));
    ASSERT_EQ(Time(22, 0, 48), routePlanner.optimalPaths[0].getArrivalTime(2));
    ASSERT_EQ(Time(22, 0, 48), routePlanner.optimalPaths[0].getDepartureTime(2));
    ASSERT_EQ("Parallel2 / Slow 3", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(3)));
    ASSERT_EQ(Time(22, 1, 31), routePlanner.optimalPaths[0].getArrivalTime(3));
    ASSERT_EQ(Time(22, 1, 31), routePlanner.optimalPaths[0].getDepartureTime(3));
    ASSERT_EQ("Parallel3 / Slow 4", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(4)));
    ASSERT_EQ(Time(22, 3, 46), routePlanner.optimalPaths[0].getArrivalTime(4));
    ASSERT_EQ(Time(22, 9, 0), routePlanner.optimalPaths[0].getDepartureTime(4));
    ASSERT_EQ("Targetstation", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(5)));
    ASSERT_EQ(Time(22, 11, 0), routePlanner.optimalPaths[0].getArrivalTime(5));
  }


  // Third Test Set: Trying to construct some Tests based on errors shown
  // during development (i.e. for an error that was found
  // construct a test that should fail if the rror is not fixed)
  {
    /*TODO(Robin): Couldn't get it to fail...
    // Test: trip where there is a chance to wait more than one day and then
    // continue on the next day, however only looking at arrival times
    // you do not see the difference in days...
     TransitNetworkDependent transitNetwork;
    string testDirectory = "./GTFSMultiDependentTestErrors2";
    int status = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG | S_IROTH |
                                              S_IXOTH);
    if (status != 0)
    {
      printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
      // make test crash if it directory could not be created...
      ASSERT_TRUE(false);
    }
    string calendarFileName =  testDirectory + "/calendar.txt";
    FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
    fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                          "friday,saturday,sunday,start_date,end_date\n"
                          "FW,1,1,1,1,1,1,1,20060701,20060731\n");
    fclose(calendarFile);

    string tripFileName =  testDirectory + "/trips.txt";
    FILE* tripFile = fopen(tripFileName.c_str(), "w");
    fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                          "A,FW,FIRSTTRIP,Downtown,2\n"
                          "A,FW,SECONDTRIP,Downtown,2\n"
                          "A,FW,THIRDTRIP,Downtown,2\n"
                          "A,FW,NEXTDAYTRIP,Downtown,2\n");

    fclose(tripFile);

    string stopsFileName = testDirectory + "/stops.txt";
    FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
    fprintf(stopsFile,
      "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
      "location_type,parent_stop\n"
      "FIRST1,Startstation,Pizza :),37.728631,-122.431282,,\n"
      "FIRST2_SECOND1,FIRST2_SECOND1 / Slow 1,Pizza :),37.728631,-122.431282,,\n"
      "SECOND2,SECOND2, Zu U9 :),52.74103,-122.422482,,\n"
      "SECOND3_NEXT1,SECOND3_NEXT1,Mein Zuhause :),37.728631,-122.433282,,\n"
      "SECOND4,SECOND4,was will man da?,37.75223,-122.418581,,\n"
      "NEXT2,NEXT2,was will man da?,37.75223,-122.418581,,\n"
      "TARGET,Targetstation,hmmm,37.761829,-122.419382,,\n");
    fclose(stopsFile);

    string stopTimesFileName = testDirectory + "/stop_times.txt";
    FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
    fprintf(stopTimesFile,
      "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
      "pickup_type,dropoff_type\n"
      "FIRSTTRIP,06:43:22,6:45:22,FIRST1,1,0,0,0\n"
      "FIRSTTRIP,6:46:0,6:47:0,FIRST2_SECOND1,2,0,0,0\n"
      "SECONDTRIP,6:50:08,6:50:48,FIRST2_SECOND1,1,0,0,0\n"
      "SECONDTRIP,6:52:08,6:54:48,SECOND2,2,0,0,0\n"
      "SECONDTRIP,6:58:00,6:59:31,SECOND3_NEXT1,3,0,0,0\n"
      "THIRDTRIP,7:7:46,7:8:46,SECOND3_NEXT1,1,0,0,0\n"
      "THIRDTRIP,7:9:46,7:10:46,SECOND4,2,0,0,0\n"
      "THIRDTRIP,7:11:46,7:12:46,TARGET,3,0,0,0\n"
      "NEXTDAYTRIP,6:27:0,6:28:0,SECOND3_NEXT1,1,0,0,0\n"
      "NEXTDAYTRIP,7:0:0,7:1:0,NEXT2,2,0,0,0\n"
      "NEXTDAYTRIP,7:1:20,7:3:0,TARGET,3,0,0,0\n");
    fclose(stopTimesFile);

    transitNetwork.parseFromGTFSFiles(testDirectory);
    remove(calendarFileName.c_str());
    remove(tripFileName.c_str());
    remove(stopsFileName.c_str());
    remove(stopTimesFileName.c_str());
    status = rmdir(testDirectory.c_str());
    if (status != 0)
      printf("couldn't remove directory %s\n", testDirectory.c_str());
    MulticriteriaDependentDijkstra routePlanner(&transitNetwork);
    routePlanner.computeShortestPath("Startstation",
                                     Time(5, 27, 10),
                                     "Targetstation",
                                     true);

    // trip should just go at 5:32:17 from berliner strasse to 5:34:13 at
    // guentzelstrasse
    ASSERT_EQ(1, routePlanner.optimalPaths.size());
    std::cout << routePlanner.optimalPaths[0].toString();
    ASSERT_EQ(1, routePlanner.optimalPaths[0].getTransfers());
    //ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
    ASSERT_EQ("Startstation", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(0)));
    ASSERT_EQ(Time(6, 45, 22), routePlanner.optimalPaths[0].getDepartureTime(0));
    ASSERT_EQ("FIRST1", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(1)));
    ASSERT_EQ(Time(22, 0, 0), routePlanner.optimalPaths[0].getArrivalTime(1));
    ASSERT_EQ(Time(22, 0, 0), routePlanner.optimalPaths[0].getDepartureTime(1));
    ASSERT_EQ("Slowtrip2", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(2)));
    ASSERT_EQ(Time(22, 0, 48), routePlanner.optimalPaths[0].getArrivalTime(2));
    ASSERT_EQ(Time(22, 0, 48), routePlanner.optimalPaths[0].getDepartureTime(2));
    ASSERT_EQ("Parallel2 / Slow 3", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(3)));
    ASSERT_EQ(Time(22, 1, 31), routePlanner.optimalPaths[0].getArrivalTime(3));
    ASSERT_EQ(Time(22, 1, 31), routePlanner.optimalPaths[0].getDepartureTime(3));
    ASSERT_EQ("Parallel3 / Slow 4", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(4)));
    ASSERT_EQ(Time(22, 3, 46), routePlanner.optimalPaths[0].getArrivalTime(4));
    ASSERT_EQ(Time(22, 9, 0), routePlanner.optimalPaths[0].getDepartureTime(4));
    ASSERT_EQ("Targetstation", transitNetwork.getStopName(
            routePlanner.optimalPaths[0].getStationId(5)));
    ASSERT_EQ(Time(22, 11, 0), routePlanner.optimalPaths[0].getArrivalTime(5));
     */
  }
}
