// Copyright November 23, 2011 Robin Schirrmeister and Simon Skilevic
// Chair of Algorithms and Datastructures Uni Freiburg



#include <queue>
#include <utility>
#include <iostream>
#include <numeric>
#include <sstream>
#include <unordered_set>
#include <utility>
#include <algorithm>
#include "./MultiCriteriaHybridDijkstra.h"
#include "./RoadNetwork.h"

using std::priority_queue;
using std::pair;
using std::stringstream;
using std::unordered_set;
using std::pair;
// _____________________________________________________________________________
MultiCriteriaHybridDijkstra::MultiCriteriaHybridDijkstra(
                                        HybridTransitRoadNetwork* hybridNetwork)
{
  this->hybridNetwork = hybridNetwork;
}

// _____________________________________________________________________________
MultiCriteriaHybridDijkstra::~MultiCriteriaHybridDijkstra()
{
}
// _____________________________________________________________________________
double MultiCriteriaHybridDijkstra::computeShortestPath(string startStop,
                           Time time, string endStop, bool buildPath)
{
  return computeShortestPath(hybridNetwork->getStopIdByTheStopName(startStop),
                             TRANSITNETWORK,
                             time,
                             hybridNetwork->getStopIdByTheStopName(endStop),
                             TRANSITNETWORK,
                             buildPath);

}
// _____________________________________________________________________________
void MultiCriteriaHybridDijkstra::printStatistic(int explored, int expanded,
                                                 int pushed, int poped)
{
  vector <int> expandedByStop;
  vector <int> expandedByRoadNode;
  int numRoadNodes = hybridNetwork->getNumStops(ROADNETWORK);
  int numStops = hybridNetwork->getNumStops(TRANSITNETWORK);
  expandedByStop.resize(numStops);
  expandedByRoadNode.resize(numRoadNodes);
  int numRoadLabels = 0;
  int numTransitLabels = 0;
  int numRemovedLabels = 0;
  int numVisitedRoadNodes = 0;
  int numVisitedStops = 0;
  for (size_t i = 0; i < labels.size(); i++)
  {
    if (labels[i].netType == ROADNETWORK)
    {
      expandedByRoadNode[labels[i].NodeId]+=1;
      numRoadLabels++;
    }
    else if (labels[i].netType == TRANSITNETWORK)
    {
      expandedByStop[hybridNetwork->getLocationId(TRANSITNETWORK,
                                                  labels[i].NodeId)]+=1;
      numTransitLabels++;
    }
    if (labels[i].removed == true)
      numRemovedLabels++;
  }
  for (size_t i = 0; i < expandedByRoadNode.size(); i++)
  {
    if (expandedByRoadNode[i] != 0)
      numVisitedRoadNodes++;
  }
  for (size_t i = 0; i < expandedByStop.size(); i++)
  {
    if (expandedByStop[i] != 0)
      numVisitedStops++;
  }
  stringstream statisticStr(stringstream::in | stringstream::out);
  statisticStr  << "STATISTICS:\n"
                << "RoadNodes:                        " << numRoadNodes << "\n"
                << "Stops:                            " << numStops << "\n"
                << "RoadNodes visited:                " << numVisitedRoadNodes << "\n"
                << "Stops visited:                    " << numVisitedStops << "\n"
                << "Labels:                           " << labels.size() << "\n"
                << "Labels pushed in the queue:       " << pushed << "\n"
                << "Labels explored:                  " << explored << "\n"
                << "Labels expanded:                  " << expanded << "\n"
                << "Labels removed:                   " << numRemovedLabels << "\n"
                << "Roadlabels:                       " << numRoadLabels << "\n"
                << "TransitLabels:                    " << numTransitLabels << "\n"
                << "Labels per RoadNode:              " << (double)numRoadLabels/(double)numVisitedRoadNodes << "\n"
                << "Labels per Stop:                  " << (double)numTransitLabels/(double)numVisitedStops << "\n";
  statistic = statisticStr.str();
  //statisticStr >> statistic;
  std::cout << statistic;
  // now replace line endings by br
  size_t lineEndingPos = statistic.find("\n");
  while(lineEndingPos != string::npos)
  {
    statistic.replace(lineEndingPos, 1, "<br />");
    lineEndingPos = statistic.find("\n", lineEndingPos + 1);
  }

}
// _____________________________________________________________________________
double MultiCriteriaHybridDijkstra::computeShortestPath(int startId,
                                                      NetworkType startNodeType,
                                                      const Time& startTime,
                                                      int endId,
                                                      NetworkType endNodeType,
                                                      bool reconstructPath)
{
  optimalPaths.clear();
  if (hybridNetwork->isExpanded() && startNodeType == TRANSITNETWORK)
    hybridNetwork->addDummyNode(startId, startTime);
  bool statistic = true;
  if (startId == -1)
  {
    printf("start stop is not known\n");
    return -1;
  }
  if (endId == -1)
  {
    printf("end stop is not known\n");
    return -1;
  }
  if (startId == endId && startNodeType == endNodeType)
  {
    printf("Start Id equal to end id => Trying to get route from one point"
            "to itself\n");
    return 0;
  }
  endParitySet = LabelSet(&labels, 0);
  bool usingExpandedNetwork = hybridNetwork->isExpanded();
  // get start nodes either from road, dependent or from expanded network
  // push labels into queue
  // get start label out of queue, get nodeid,
  // check if we arrived at end station,
  // otherwise call getconnections for node id and time
  // of the corresponding network
  // push all endnodes of the corresponding connections as new labels
  priority_queue <pair<size_t, int >,
                  std::vector<pair<size_t, int > >,
                  std::greater<pair<size_t, int > > >
                  _openNodeQueue;
  int expanded = 0;
  int explored = 0;
  int pushed = 0;
  int poped = 0;
  int transferTime = 4 * 60;
  if (hybridNetwork->isExpanded())
    transferTime = 0;
  transitLabelSets.clear();
  roadLabelSets.clear();
  transitLabelSets.resize(hybridNetwork->getNumNodes(TRANSITNETWORK),
                   LabelSet(&labels, transferTime));
  roadLabelSets.resize(hybridNetwork->getNumNodes(ROADNETWORK),
                   LabelSet(&labels, 0));
  labels.clear();

  vector<LiveConnection> startConnections;
  hybridNetwork->getStartConnections(startId, startNodeType, startTime,
                                     &startConnections);
  int newNodeId;
  vector<int> newCriterias;
  bool isMultiCriteria = true;
  for (size_t i = 0; i < startConnections.size(); i++)
  {
    explored++;
    const LiveConnection& currentLiveConnection = startConnections[i];
    newNodeId = currentLiveConnection.toNodeId;
    newCriterias.clear();
    int zeroTransfers = 0;
    int expandedFromLabelId = -1;
    newCriterias.push_back(currentLiveConnection.time);
    newCriterias.push_back(zeroTransfers);
    assert(currentLiveConnection.destinationType == TRANSITNETWORK
            || currentLiveConnection.destinationType == ROADNETWORK);
    if (currentLiveConnection.destinationType == TRANSITNETWORK)
    {
      bool atEndStation = hybridNetwork->equalWithEndStation(endNodeType, endId,
                                            currentLiveConnection.destinationType,
                                            newNodeId);
      if (transitLabelSets[newNodeId].AddLabel(currentLiveConnection.connection,
                                    newCriterias,
                                    newNodeId, expandedFromLabelId,
                                    currentLiveConnection.destinationType,
                                    usingExpandedNetwork,
                                    atEndStation, true, true))
        {
        pair<int, int > weightAndLabelId(currentLiveConnection.time,
                                                labels.size()-1);
            _openNodeQueue.push(weightAndLabelId);
            pushed++;
        }
    }
    else
    {
      bool atEndStation = true;
      if (roadLabelSets[newNodeId].AddLabel(currentLiveConnection.connection,
                                    newCriterias,
                                    newNodeId, expandedFromLabelId,
                                    currentLiveConnection.destinationType,
                                    true,
                                    atEndStation, true, true))
        {
        pair<int, int > weightAndLabelId(currentLiveConnection.time,
                                                labels.size()-1);
            _openNodeQueue.push(weightAndLabelId);
            pushed++;
        }
    }    
  }
  int currentNodeId = startId;
  int currentLabelId;
  Connection currentConnection;
  int sumOfNewCriterias;
  bool dijkstraFinished = _openNodeQueue.empty();
  NetworkType currentType;

  // Now starts the Dijkstra...
  while (!dijkstraFinished)
  {
    // get minimum of node queue
    currentLabelId = _openNodeQueue.top().second;
    currentNodeId = labels[currentLabelId].NodeId;
    currentConnection = labels[currentLabelId].connection;
    assert( currentConnection.arrivalTime == labels[currentLabelId].connection.arrivalTime);
    assert( currentConnection.departureTime == labels[currentLabelId].connection.departureTime);
    currentType = labels[currentLabelId].netType;
   
    _openNodeQueue.pop();
    poped++;
    // check if already explored
    if (labels[currentLabelId].removed == true)
    {
       dijkstraFinished = _openNodeQueue.empty();
       continue;
    }
    if (endParitySet._expandedFrom.size() != 0)
    {
      int transferTime = 0;
      bool atEndStation = true;
      // should be false now beacuse we do not need the transfers to
      // be different by more than one and we can ignore transfer time..
      if (!endParitySet.labelMustBeAdded(
              labels[currentLabelId].criterias[0],
              labels[currentLabelId].criterias[1],
              labels[currentLabelId].connection.lineId,
              transferTime,
              usingExpandedNetwork, atEndStation, isMultiCriteria))
      {
        labels[currentLabelId].removed = true;
        dijkstraFinished = _openNodeQueue.empty();
        continue;
      }
    }
    bool isEndNode = hybridNetwork->equalWithEndStation(endNodeType, endId,
                                                        currentType, currentNodeId);
    if (isEndNode)
    {      
      // add to end parity set and continue dijkstra with next label...
      bool couldAddLabelToParitySet =
        endParitySet.AddLabel(labels[currentLabelId].connection,
              labels[currentLabelId].criterias, currentNodeId, currentLabelId,
              labels[currentLabelId].netType,
              true, true, false, true);
      assert(couldAddLabelToParitySet);
      dijkstraFinished = _openNodeQueue.empty();
      expanded++;
      continue;
    }
    // get outgoing edges from this node
    bool isStartNode = hybridNetwork->equalWithStartStation(startNodeType,
                                                            startId,
                                                            currentType,
                                                            currentNodeId);

    bool isMultiCriteria = true;
    
    vector<LiveConnection> outConnections;
    // TODO:
    expanded++;
    hybridNetwork->getConnections(currentNodeId, currentType,
                                  endId, endNodeType,
                               currentConnection.arrivalTime,
                               currentConnection.lineId,
                               isMultiCriteria, isStartNode,
                               &outConnections);
    for (size_t i = 0; i < outConnections.size(); i++)
      if (labels[currentLabelId].expandedFromLabelId == -1
              || outConnections[i].toNodeId
                  != labels[labels[currentLabelId].expandedFromLabelId].NodeId)
      {
        explored++;
        LiveConnection& newConnection = outConnections[i];
        vector<int> newCriterias = labels[currentLabelId].criterias;
        newCriterias[0] += newConnection.time;
        newCriterias[1] += newConnection.transfers;
        int newNodeId = newConnection.toNodeId;
        bool atEndStation = hybridNetwork->equalWithEndStation(endNodeType, endId,
                newConnection.destinationType,
                newNodeId);
        if (atEndStation && newConnection.destinationType == TRANSITNETWORK
                && currentType == ROADNETWORK)
          newCriterias[1] -= newConnection.transfers;
        assert(newConnection.destinationType == TRANSITNETWORK
                || newConnection.destinationType == ROADNETWORK);
        if (newConnection.destinationType == TRANSITNETWORK)
        {
          if (transitLabelSets[newNodeId].AddLabel(newConnection.connection,
                  newCriterias, newNodeId, currentLabelId,
                  newConnection.destinationType,
                  usingExpandedNetwork,
                  atEndStation, true, true))
          {
            sumOfNewCriterias = std::accumulate(newCriterias.begin(),
                    newCriterias.end(),
                    0.0);
            _openNodeQueue.push(pair<int, int>(sumOfNewCriterias,
                    labels.size() - 1));
            pushed++;
          }
        } else
        {         
          if (roadLabelSets[newNodeId].AddLabel(newConnection.connection,
                  newCriterias, newNodeId, currentLabelId,
                  newConnection.destinationType,
                  true,
                  true, true, true))
          {
            sumOfNewCriterias = std::accumulate(newCriterias.begin(),
                    newCriterias.end(),
                    0.0);            
            _openNodeQueue.push(pair<int, int>(sumOfNewCriterias,
                    labels.size() - 1));
            pushed++;
          }
        }
      }
    dijkstraFinished = _openNodeQueue.empty();
  }
  printf("outside of dijkstra\n");
  if (statistic)
    printStatistic(explored, expanded, pushed, poped);
  if (endParitySet._expandedFrom.size() == 0)
  {
    printf("shortest Path not found from %d to %d\n", startId, endId);
    if (hybridNetwork->isExpanded() && startNodeType == TRANSITNETWORK)
      hybridNetwork->removeDummyNode();
    return -1;
  }
  std::cout << "Endlabels:";
  for (size_t i = 0; i < endParitySet._expandedFrom.size(); i++)
  {
    for (size_t j = 0; j < endParitySet._expandedFrom[i].size(); j++)
    {
      const Label& endLabel = labels[endParitySet._expandedFrom[i][j]];
      std::cout << "\ntime: " << endLabel.criterias[0];
      std::cout << "\ntransfers: " << endLabel.criterias[1];
      std::cout << "\nweight: " << endLabel.criterias[0] + endLabel.criterias[1];
    }
  }
  std::cout << "\n";
  if (reconstructPath)
  {
    if (!hybridNetwork->isExpanded())
    {
      reconstructShortestPath(startTime, startId);
    }
    else
    {
      unordered_set<int> startSet;
      for (size_t i = 0; i < startConnections.size(); i++)
        startSet.insert(startConnections[i].toNodeId);
      reconstructShortestPath(startTime, startId);
      if (hybridNetwork->isExpanded() && startNodeType == TRANSITNETWORK)
        hybridNetwork->removeDummyNode();
    }
  }
  // TODO(Robin): make it return actual length...
  return 0;
}

//______________________________________________________________________________
string MultiCriteriaHybridDijkstra::createDijkstraStatisticsJSON()
{

  stringstream jsonStream;
  unordered_set<int> roadSet;
  unordered_set<int> transitSet;
  // first the stations that were touched in this route and their locations
  // only add each station once
  jsonStream << "{ \"statisticsText\":\n"
             << "\"" << statistic << "\" \n}\n" ;
  return jsonStream.str();
}

//______________________________________________________________________________
string MultiCriteriaHybridDijkstra::createDijkstraLocationsJSON(
                                 const unordered_set<int>& affectedLabelIds) const
{

  stringstream jsonStream;
  unordered_set<int> roadSet;
  unordered_set<int> transitSet;
  // first the stations that were touched in this route and their locations
  // only add each station once
  jsonStream << "{ \"propertyKeys\":\n"
          "        {\"type\": 0,\n"
          "        \"locationId\": 1,\n"
          "        \"longitude\": 2,\n"
          "        \"latitude\": 3"
          "},\n"
          "\"usedLocations\": \"[";

  bool firstLabelAdded = false;  // for comma
  for (size_t labelId = 0; labelId < labels.size(); labelId++)
  {
    if (affectedLabelIds.count(labelId) == 0)
      continue;
    const Label& curLabel = labels[labelId];
    int nodeId = curLabel.NodeId;
    NetworkType type = curLabel.netType;
    int locationId = hybridNetwork->getLocationId(type, nodeId);
    if (type == TRANSITNETWORK && transitSet.count(locationId) != 0)
      continue;
    else if (type == ROADNETWORK && roadSet.count(locationId) != 0)
      continue;
    int typeNr;
    if (type == ROADNETWORK)
      typeNr = 0;
    else
      typeNr = 1;
    double latitude = hybridNetwork->getLatitude(type, nodeId);
    double longitude = hybridNetwork->getLongitude(type, nodeId);
    if (firstLabelAdded) jsonStream << ",";
    jsonStream << "["
               << typeNr << ","
               << locationId << ","
               << longitude << ","
               << latitude
               << "]";
    if (type == TRANSITNETWORK)
      transitSet.insert(locationId);
    else if (type == ROADNETWORK)
      roadSet.insert(locationId);
    firstLabelAdded = true;
  }
  jsonStream << "]\"}\n";
  return jsonStream.str();
}

void MultiCriteriaHybridDijkstra::createAddedAndRemovedMaps()
{
  _labelIdToAddedLabelIds.clear();
  _labelIdToRemovedLabelIds.clear();
  for (size_t labelId = 0; labelId < labels.size(); labelId++)
  {
    const Label& curLabel = labels[labelId];
    if (curLabel.expandedFromLabelId != -1)
      _labelIdToAddedLabelIds[curLabel.expandedFromLabelId].push_back(labelId);
    if (curLabel.removedByLabelId != -1)
      _labelIdToRemovedLabelIds[curLabel.removedByLabelId].push_back(labelId);
  }
}

//TODO(Robin): redo as const function?
string MultiCriteriaHybridDijkstra::createDijkstraStepJSON(int stepNumber,
                          int expandedLabelId,
                          unordered_set<int>* labelIdsAffectedInThisStep)
{
  stringstream jsonStream;
  jsonStream << "{";
  // remember which labels were added and which were removed
  const vector<int>& addedLabelIds = _labelIdToAddedLabelIds[expandedLabelId];
  const vector<int>& removedLabelIds = _labelIdToRemovedLabelIds
                                                [expandedLabelId];
  jsonStream << "\"stepNr\" : " << stepNumber << ","
             << "\"expandedId\" : " << expandedLabelId << ","
             << "\"addedIds\" : [";
  for (size_t i = 0; i < addedLabelIds.size(); i++)
  {
    if (i > 0) jsonStream << ",";
    jsonStream << addedLabelIds[i];
    labelIdsAffectedInThisStep->insert(addedLabelIds[i]);
  }
  jsonStream << "],"
             << "\"removedIds\" : [";
  for (size_t i = 0; i < removedLabelIds.size(); i++)
  {
    if (i > 0) jsonStream << ",";
    jsonStream << removedLabelIds[i];
    labelIdsAffectedInThisStep->insert(removedLabelIds[i]);
  }
  jsonStream << "]"
             << "}";
  return jsonStream.str();
}
string MultiCriteriaHybridDijkstra::createDijkstraStepsJSON(int startStep,
                                                     int endStep)
{
  // TODO(Robin): redo this always giving back one step?
  assert(startStep >= 0);
  assert(endStep > 0);
  stringstream jsonStream;
  // first the stations that were touched in this route and their locations
  // only add each station once
  // Sort the labels in the order they were expanded by sorting
  // them by their weight
  int numberOfExpandedLabelsInDijkstra = 0;
  vector<pair< int,int> > weightAndExpandedLabelIdsSorted;
  for (size_t i = 0; i < labels.size(); i++)
  {
    const Label& curLabel = labels[i];
    if (!curLabel.removed)
    {
      numberOfExpandedLabelsInDijkstra++;
      weightAndExpandedLabelIdsSorted.push_back(pair<int, int>(
                                                     labels[i].criterias[0] +
                                                     labels[i].criterias[1],
                                                     i));
    }
  }
  if (endStep >= numberOfExpandedLabelsInDijkstra)
    endStep = numberOfExpandedLabelsInDijkstra - 1;
  // remember which labels were expanded in these steps
  vector<int> expandedLabelIds;
  unordered_set<int> expandedLabelSet;
  unordered_set<int> labelIdsAffectedInThisStep;
  sort(weightAndExpandedLabelIdsSorted.begin(),
       weightAndExpandedLabelIdsSorted.end());
  for (size_t i = startStep; i <= static_cast<size_t>(endStep); i++)
  {
    int currentlyExpandedLabelId = weightAndExpandedLabelIdsSorted[i].second;
    expandedLabelIds.push_back(currentlyExpandedLabelId);
    expandedLabelSet.insert(currentlyExpandedLabelId);
    labelIdsAffectedInThisStep.insert(currentlyExpandedLabelId);
  }

  jsonStream << "{"
             << "\"numberOfExpandedLabelsInDijkstra\": " 
             << numberOfExpandedLabelsInDijkstra
             << ",\n"
             << "\"dijkstraSteps\": " << "[";
  for (size_t i = 0; i < expandedLabelIds.size(); i++)
  {
    if (i > 0) jsonStream << ",";
    int stepNumber = startStep + i;
    jsonStream << createDijkstraStepJSON(stepNumber,
                                         expandedLabelIds[i],
                                         &labelIdsAffectedInThisStep);
  }
  jsonStream << "],"
             <<   "\"dijkstraLocations\": "
             << createDijkstraLocationsJSON(labelIdsAffectedInThisStep)
             << ",\n";
  // TODO(Robin): make this better by having a function in the label
  // that returns the indices?
  // see createCompressedJson to see if these indices are correct
  int labelIdIndex = 0;
  int typeIndex = 1;
  int locationIdIndex = 2;
  int nodeIdIndex = 3;
  int tripIdIndex = 4;
  int arrivalHourIndex = 5;
  int arrivalMinuteIndex = 6;
  int arrivalSecondIndex = 7;
  int weightIndex = 8;
  int timeIndex = 9;
  int transferIndex = 10;
  int expandedFromIndex = 11;
  int removedByIndex = 12;
  jsonStream << "  \"propertyKeys\": {"
             << "  \"id\": "<< labelIdIndex << ",\n"
             << "  \"type\": "<< typeIndex << ",\n"
             << "  \"locationId\": "<< locationIdIndex << ",\n"
             << "  \"nodeId\": "<< nodeIdIndex << ",\n"
             << "  \"tripId\": "<< tripIdIndex << ",\n"
             << "  \"arrivalHour\": "<< arrivalHourIndex << ",\n"
             << "  \"arrivalMinute\": "<< arrivalMinuteIndex << ",\n"
             << "  \"arrivalSecond\": "<< arrivalSecondIndex << ",\n"
             << "  \"weight\": "<< weightIndex << ",\n"
             << "  \"time\": "<< timeIndex << ",\n"
             << "  \"transfers\": "<< transferIndex << ",\n"
             << "  \"expandedFromLabelId\": "<< expandedFromIndex << ",\n"
             << "  \"removedByLabelId\": "<< removedByIndex << "\n"
             << "  },\n"
             << "  \"labels\": \"[";
  bool firstLabelAdded = false;  // for the placement of the comma
  for (size_t labelId = 0; labelId < labels.size(); labelId++)
  {
    if (labelIdsAffectedInThisStep.count(labelId) == 0)
      continue;
    const Label& curLabel = labels[labelId];
    Time arrivalTime;
    int locationId;
    NetworkType netType = curLabel.netType;
    if (hybridNetwork->isExpanded() && curLabel.netType == TRANSITNETWORK)
    {
      arrivalTime = hybridNetwork->getNode(netType, curLabel.NodeId)->time;
    }
    else
    {
      arrivalTime = curLabel.connection.arrivalTime;
    }
    locationId = hybridNetwork->getLocationId(netType,  curLabel.NodeId);
    if (firstLabelAdded) jsonStream << ",";
    jsonStream << curLabel.createCompressedJSON(labelId, arrivalTime,
                                                locationId);
    firstLabelAdded = true;
  }
  jsonStream << "]\"\n";
  jsonStream << "}\n";
  return jsonStream.str();

}
// _____________________________________________________________________________
void MultiCriteriaHybridDijkstra::reconstructShortestPath(const Time& startTime,
                                                          int startID)
{
  //assert(hybridNetwork->isExpanded());
  int nodeId;
  int labelId;
  // TODO(Robin): redo this well...
  // reconstruct the path, leave out transfer nodes!
  for (size_t i = 0; i < endParitySet._expandedFrom.size(); i++)
    for (size_t j = 0; j < endParitySet._expandedFrom[i].size(); j++)
    {
      TransitPath newPath(startTime);
      labelId = endParitySet._expandedFrom[i][j];
      Time departureTime;
      while (labelId != -1)
      {
        int newLabelId = labels[labelId].expandedFromLabelId;
        while (newLabelId != -1
            && hybridNetwork->getLocationId(labels[newLabelId].netType, 
                                            labels[newLabelId].NodeId)
               == hybridNetwork->getLocationId(labels[labelId].netType,
                                               labels[labelId].NodeId))
        {
          labelId = newLabelId;
          newLabelId = labels[labelId].expandedFromLabelId;
        }
        Time arrivalTime;
        nodeId = labels[labelId].NodeId;
        arrivalTime = labels[labelId].connection.arrivalTime;
        NetworkType netType = labels[labelId].netType;
        TransitPath::PathNodeType pathNodeType;
        if (netType == TRANSITNETWORK)
          pathNodeType = TransitPath::TRANSITNODE;
        else
          pathNodeType = TransitPath::ROADNODE;
        double latitude = hybridNetwork->getLatitude(netType, nodeId);
        double longitude = hybridNetwork->getLongitude(netType, nodeId);
        int locationId = hybridNetwork->getLocationId(netType,
                                                      nodeId);
        int tripId = labels[labelId].connection.lineId;
        if (locationId == startID)
          arrivalTime = startTime;
        newPath.addStation(pathNodeType,
                           locationId, latitude, longitude,
                           arrivalTime, departureTime,
                           tripId);
        departureTime = labels[labelId].connection.departureTime;
        labelId = newLabelId;
      }
      newPath.reverse();
      optimalPaths.push_back(newPath);
    }
}
