#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include <bits/stl_vector.h>
#include "./Timer.h"
#include "./Dijkstra.h"
#include "./EdgeDependent.h"
#include "./TransitNetworkDependent.h"
#include "./HybridTransitRoadNetwork.h"
#include "./DependentDijkstra.h"
#include "./MulticriteriaDependentDijkstra.h"
#include "MulticriteriaExpandedDijkstra.h"
#include "MultiCriteriaHybridDijkstra.h"

using std::string;
using std::vector;
using std::cout;
using std::endl;
unsigned int randomSeedPerf;
// _____________________________________________________________________________
void printUsage()
{
  printf("This program measures performance for shortest path queries "
         "transit networks.\n\n"
         "USAGE: ./TransitNetworkDependentPerf GTFSFolder OSMFile\n");
}
// _____________________________________________________________________________
int getRandomNodeId(int size)
{
  int idToTry = rand_r(&randomSeedPerf) % size;
  return idToTry;
}
void showDifferentPaths(const vector<TransitPath>& dependentPaths,
                        const vector<TransitPath>& expandedPaths)
{
  // check if we found Different Paths, if yes diff everything
    // with everything
    pair<vector<int>, vector<int> > differentPaths =
                                            TransitPath::findDifferentResults(
                                            dependentPaths,
                                            expandedPaths);

    if (differentPaths.first.size()> 0
        || differentPaths.second.size() > 0)
    {
      printf("\033[34mDIFFERENT PATHS:\033[0m\n");
      const vector<int>& firstIndices = differentPaths.first;
      const vector<int>& secondIndices = differentPaths.second;
      for (size_t i = 0; i < firstIndices.size(); i++)
      {
        // if expanded found nothing nothing, just print trips if dependent
        if (secondIndices.size() == 0)
        {
          printf("\033[31mTrip %zu only found by Dependent:\033[0m\n", i);
          printf("%s\n\n", dependentPaths[firstIndices[i]].toString().c_str());
        }
        for (size_t j = 0; j < secondIndices.size(); j++)
        {
          const TransitPath& firstPath = dependentPaths[firstIndices[i]];
          const TransitPath& secondPath = expandedPaths[secondIndices[j]];
          // print inr ed if trips have different transfers or different
          // duration else just print blue
          if (firstPath.getDurationInMinutes() !=
              secondPath.getDurationInMinutes() ||
              firstPath.getTransfers() != secondPath.getTransfers())
          {
            printf("\033[31m");
          }
          else
          {
            printf("\033[34m");
          }
          printf("DIFF of trips %d | %d (Dependent | Expanded ):\n"
                 "\033[0m", firstIndices[i], secondIndices[j]);
          printf("%s\n\n", firstPath.createDiffTo(secondPath).c_str());
        }
      }
      // if dependent found nothing, just print trips of expanded
      if (firstIndices.size() == 0)
      {
        for (size_t i = 0; i < secondIndices.size(); i++)
        {
          printf("\033[31mTrip %zu only found by Expanded:\033[0m\n", i);
          const TransitPath& secondPath = expandedPaths[secondIndices[i]];
          printf("%s\n\n", secondPath.toString().c_str());
        }
      }
    printf("---------------------------------------------------------------\n");
    printf("Dependent\n\n");
    for (size_t i = 0; i < dependentPaths.size(); i++)
    {
      printf("%s\n", dependentPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    printf("---------------------------------------------------------------\n");
    printf("Expanded\n\n");
    for (size_t i = 0; i < expandedPaths.size(); i++)
    {
      printf("%s\n", expandedPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    }
}
// _____________________________________________________________________________
int main(int argc, char** argv)
{
  string GTFSFolder;
  // Needed for getopt_long.
  int optChr;
  int numberOfQueries;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that GTFS folder was given
  if (argc > 3)
  {
    printUsage();
    return 1;
  }
  // TODO(Robin): redo with one weekday only...
  char testDay = 0;
  int transferTime = 4 * 60;
  GTFSFolder = argv[optind];
  string OSMFile;
  if (argc == 3)
    OSMFile =  argv[argc - 1];
  // parse transit network
  TransitNetworkDependent transitNetworkDependent;
  RoadNetwork roadNetwork;
  cout << "Parsing OSM File:" << endl;
  transitNetworkDependent.parseFromGTFSFiles(GTFSFolder);
  cout << " done." << endl;
  roadNetwork.readFromOsmFile(OSMFile);
  cout << "Parsing GTFS File by dependent network:" << endl;
  transitNetworkDependent.parseFromGTFSFiles(GTFSFolder);
  cout << " done." << endl;
  HybridTransitRoadNetwork hybridNetworkDependent(&roadNetwork, &transitNetworkDependent, NULL);

  cout << "Parsing GTFS File by expanded network:" << endl;
  TransitNetworkExpanded transitNetworkExpanded;
  transitNetworkExpanded.setVerbosity(2);
  transitNetworkExpanded.parseFromGTFSFiles(GTFSFolder, testDay, transferTime);
  cout << " done." << endl;
  HybridTransitRoadNetwork hybridNetworkExpanded(&roadNetwork, NULL, &transitNetworkExpanded);
  //MulticriteriaDependentDijkstra routePlannerDependent(&transitNetworkDependent);
  MultiCriteriaHybridDijkstra routePlannerHybridDependent(&hybridNetworkDependent);
//  cout << "Bulding of HybridNetwork by dependent:" << endl;
//
//  cout << " done." << endl;
  MultiCriteriaHybridDijkstra routePlannerHybridExpanded(&hybridNetworkExpanded);
  cout << "Building Hybridnetwork with dependent network:" << endl;
  hybridNetworkDependent.buildHybridNetwork(4);
  cout << " done." << endl;
  cout << "Building Hybridnetwork with expanded network:" << endl;
  hybridNetworkExpanded.buildHybridNetwork(4);
  cout << " done." << endl;
  // TODO(Robin): also allow direct use(?)...
  numberOfQueries = 100;
  //int startId, endId;
 // size_t settledNodes = 0;
  vector< vector <int> > randomInput;
  randomInput.resize(numberOfQueries);
  cout << "Creating of test data:" << endl;
  for (int i = 0; i < numberOfQueries; i++)
  {
    int startId = getRandomNodeId(transitNetworkDependent.getNumNodes());

    int endId = startId;
    while (startId == endId)
      endId = getRandomNodeId(transitNetworkDependent.getNumNodes());
    int hour = getRandomNodeId(24);
    int min = getRandomNodeId(60);
    int sec = getRandomNodeId(60);
    randomInput[i].push_back(startId); // 0
    randomInput[i].push_back(endId);   // 1
    randomInput[i].push_back(hour);    // 2
    randomInput[i].push_back(min);     // 3
    randomInput[i].push_back(sec);     // 4
  }

  cout << endl << "done." << endl;

  cout << "Testing of dependent Dijkstra on hybridnetwork:" << endl;
  double timeSum = 0;
  Timer timer;
  for (int i = 0; i < numberOfQueries; i++)
  {
    int startStopId = randomInput[i][0];
    int endStopId = randomInput[i][1];
    Time startTime(randomInput[i][2], randomInput[i][3], randomInput[i][4]);
    timer.start();
    routePlannerHybridDependent.computeShortestPath(startStopId, TRANSITNETWORK,
                                             startTime,
                                             endStopId, TRANSITNETWORK, false);
    timer.stop();
    timeSum += timer.usecs();
  }
  printf("Time taken for dependent Dijkstra on hybridnetwork : %7.2f ms\n",
                                               timeSum / 1000.0);
  printf("Average Time: %7.2f ms\n",
          timeSum / (1000.0 * numberOfQueries));

  cout << "Testing of expanded Dijkstra on hybridnetwork:" << endl;
  timeSum = 0;
  for (int i = 0; i < numberOfQueries; i++)
  {
    int startStopId = randomInput[i][0];
    int endStopId = randomInput[i][1];
    Time startTime(randomInput[i][2], randomInput[i][3], randomInput[i][4]);
    timer.start();
    routePlannerHybridExpanded.computeShortestPath(startStopId,TRANSITNETWORK,
                                                          startTime,
                                                          endStopId,
                                                          TRANSITNETWORK,
                                                          false);
    timer.stop();
    timeSum += timer.usecs();
  }
  printf("Time taken for expanded Dijkstra on hybridnetwork : %7.2f ms\n",
                                               timeSum / 1000.0);
  printf("Average Time: %7.2f ms\n",
          timeSum / (1000.0 * numberOfQueries));

  return 0;
}