// Copyright November 8, 2011 Robin Schirrmeister and Simon Skilevic
// Chair of Algorithms and Datastructures Uni Freiburg
/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 * Testclass for Transit network
 */

#include <sys/stat.h>
#include <stdio.h>
#include <string>
#include <unordered_map>
#include "./MultiCriteriaHybridDijkstra.h"

using std::string;
using std::unordered_map;
TEST(MultiCriteriaHybridTest, getStartConnections)
{
  
}
TEST(MultiCriteriaHybridTest, computeShortestPath)
{
  {
   TransitNetworkDependent transitNetwork;

  string testDirectory = "./GTFSMultiDependentTest";
  int status = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG | S_IROTH |
                                            S_IXOTH);
  if (status != 0)
  {
    printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
    // make test crash if it directory could not be created...
    ASSERT_TRUE(false);
  }
  string calendarFileName =  testDirectory + "/calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);

  string tripFileName =  testDirectory + "/trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n"
                        "A,FW,SHORTCUTTRANSFER,Downtown,2\n"
                        "A,WD,SHORTCUTTIME,Downtown,2\n");  // add extra trips
                                                     // for transfer tests...
  fclose(tripFile);

  string stopsFileName = testDirectory + "/stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
    "location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U9_U7_1,Berliner Strasse, Zu U9 :),33.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);
  string frequenciesFileName = testDirectory + "/frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:00,25:30:00,10800\n"
    "U7WE,00:00:00,24:00:00,5400\n"
    "U7WD,05:30:00,26:30:00,3600\n"
    "U7FW,05:29:32,25:00:00,7200\n"
    "SHORTCUTTRANSFER,05:29:32,25:00:00,7200\n"
    "SHORTCUTTIME,05:29:32,25:00:00,7200\n");
  fclose(frequenciesFile);

  string stopTimesFileName = testDirectory + "/stop_times.txt";
  FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
  fprintf(stopTimesFile,
    "trip_id,arrival_time,departure_time,stop_id,stop_sequence,"
    "pickup_type,dropoff_type\n"
    "U9FW,5:30:0,5:32:17,U9_U7_1,1,0,0,0\n"
    "U9FW,5:34:13,05:38:0,U92,2,0,0,0\n"
    // last stop transfer and departure time equal if transfer time = 4 minutes
    "U7WE,0:0:0,0:1:0,U71,1,0,0,0\n"
    "U7WE,0:4:0,0:6:0,U9_U7_1,2,0,0,0\n"
    "U7WD,5:26:0,05:26:32,U70,1,0,0,0\n"
    "U7WD,5:26:58,05:27:10,U71,2,0,0,0\n"
    "U7WD,5:27:35,05:29:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:29:32,5:30:0,U70,1,0,0,0\n"
    "U7FW,5:32:0,5:33:0,U71,2,0,0,0\n"
    "U7FW,5:34:0,5:35:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:36:0,5:37:0,U73,4,0,0,0\n"
    "SHORTCUTTRANSFER,5:29:32,5:30:0,U71,1,0,0,0\n"
    "SHORTCUTTRANSFER,5:38:0,5:40:0,U92,2,0,0,0\n"
    "SHORTCUTTIME,5:29:32,5:30:58,U71,1,0,0,0\n"
    "SHORTCUTTIME,5:32:02,5:32:0,U73,2,0,0,0\n");
  fclose(stopTimesFile);

  transitNetwork.parseFromGTFSFiles(testDirectory);

  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
  remove(stopsFileName.c_str());
  remove(frequenciesFileName.c_str());
  remove(stopTimesFileName.c_str());
  status = rmdir(testDirectory.c_str());
  if (status != 0)
    printf("couldn't remove directory %s\n", testDirectory.c_str());
  // making a shortcut from eisenacher strasse to guentzelstrasse
  RoadNetwork roadNetwork;
  roadNetwork.parseFromString(
          "Nodes: 0[37.7287.5/-122.431], 1[37.761829/-122.419382]\n"
          "Edges: (0 -> 1 | 0.1)");
  int neighboursPerStation = 4;
  // TEST : Direct Way from Berliner Strasse to Guentzelstrasse ...
  HybridTransitRoadNetwork hybridNetwork(&roadNetwork, &transitNetwork, NULL);
  hybridNetwork.buildHybridNetwork(neighboursPerStation);
  MultiCriteriaHybridDijkstra routePlanner(&hybridNetwork);
  routePlanner.computeShortestPath("Eisenacher Strasse",
                                   Time(5, 29, 31),
                                   "Guentzelstrasse",
                                   true);

  std::cout << routePlanner.optimalPaths[0].toString();
  // trip should just go at 5:32:17 from berliner strasse to 5:34:13 at
  // guentzelstrasse
  ASSERT_EQ(1, routePlanner.optimalPaths.size());
  ASSERT_EQ(2, routePlanner.optimalPaths[0].size());
  ASSERT_EQ("Berliner Strasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(0)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getDepartureTime(0).hour());
  ASSERT_EQ(32, routePlanner.optimalPaths[0].getDepartureTime(0).minute());
  ASSERT_EQ(17, routePlanner.optimalPaths[0].getDepartureTime(0).second());
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(
          routePlanner.optimalPaths[0].getStationId(1)));
  ASSERT_EQ(5, routePlanner.optimalPaths[0].getArrivalTime(1).hour());
  ASSERT_EQ(34, routePlanner.optimalPaths[0].getArrivalTime(1).minute());
  ASSERT_EQ(13, routePlanner.optimalPaths[0].getArrivalTime(1).second());
  }
}

