// Copyright 2011 Robin Schirrmeister and Simon Skilevic
// Chair of Algorithms and Datastructures Uni Freiburg


#ifndef ROUTEPLANNING_MULTICRITERIAEXPANDEDDIJKSTRA_H_
#define ROUTEPLANNING_MULTICRITERIAEXPANDEDDIJKSTRA_H_
#include <queue>
#include "./Graph.h"
#include "./TransitNetworkExpanded.h"
#include "./Label.h"
#include "./TransitPath.h"
using std::priority_queue;
class MulticriteriaExpandedDijkstra
{
public:
  MulticriteriaExpandedDijkstra(TransitNetworkExpanded* transitNetwork);
  virtual ~MulticriteriaExpandedDijkstra();
  FRIEND_TEST(MultiCriteriaExpandedTest, computeShortestPath);
  // convenience function, just calls the function below
  double computeShortestPath(string startStop, Time time,
                             string endStop, bool buildPath);
  // computes shortest path inbetween the two stops starting at the given start
  // time, returns lenght of that path..
  double computeShortestPath(int startStopId, Time startTime, int endStopId,
                           bool reconstructPath);
  // the shortest path: last Edge is a dummy edge...
  vector<pair<NodeExpanded, Edge> > transitShortestPath;
  vector <Label> labels;
  vector <LabelSet> labelSets;
  vector<TransitPath> optimalPaths;
private:
  void reconstructShortestPath(const Time& startTime,
                               const unordered_set<int>& startIds,
                               const LabelSet& endParitySet);
  TransitNetworkExpanded* _transitGraph;
  vector<int> _timesFromStart;
  priority_queue <pair<int, int>,
                    std::vector<pair<int, int> >,
                    std::greater<pair<int, int > > > _openNodeQueue;

  // stores which nodeid (index of vector)
  // was expanded from which node id (first part of pair)
  // with which edge (second part of pair)
  vector<bool> _expanded;
};

#endif  // ROUTEPLANNING_MULTICRITERIAEXPANDEDDIJKSTRA_H_

