#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include <bits/stl_vector.h>
#include "./Timer.h"
#include "./Dijkstra.h"
#include "./EdgeDependent.h"
#include "./TransitNetworkDependent.h"
#include "./DependentDijkstra.h"
#include "./MulticriteriaDependentDijkstra.h"
#include "MulticriteriaExpandedDijkstra.h"

using std::string;
using std::vector;
using std::cout;
using std::endl;
unsigned int randomSeedPerf;
// _____________________________________________________________________________
void printUsage()
{
  printf("This program measures performance for shortest path queries "
         "transit networks.\n\n"
         "USAGE: ./TransitNetworkDependentPerf GTFSFolder\n");
}
// _____________________________________________________________________________
int getRandomNodeId(int size)
{
  int idToTry = rand_r(&randomSeedPerf) % size;
  return idToTry;
}
void showDifferentPaths(const vector<TransitPath>& dependentPaths,
                        const vector<TransitPath>& expandedPaths)
{
  // check if we found Different Paths, if yes diff everything
    // with everything
    pair<vector<int>, vector<int> > differentPaths =
                                            TransitPath::findDifferentResults(
                                            dependentPaths,
                                            expandedPaths);

    if (differentPaths.first.size()> 0
        || differentPaths.second.size() > 0)
    {
      printf("\033[34mDIFFERENT PATHS:\033[0m\n");
      const vector<int>& firstIndices = differentPaths.first;
      const vector<int>& secondIndices = differentPaths.second;
      for (size_t i = 0; i < firstIndices.size(); i++)
      {
        // if expanded found nothing nothing, just print trips if dependent
        if (secondIndices.size() == 0)
        {
          printf("\033[31mTrip %zu only found by Dependent:\033[0m\n", i);
          printf("%s\n\n", dependentPaths[firstIndices[i]].toString().c_str());
        }
        for (size_t j = 0; j < secondIndices.size(); j++)
        {
          const TransitPath& firstPath = dependentPaths[firstIndices[i]];
          const TransitPath& secondPath = expandedPaths[secondIndices[j]];
          // print inr ed if trips have different transfers or different
          // duration else just print blue
          if (firstPath.getDurationInMinutes() !=
              secondPath.getDurationInMinutes() ||
              firstPath.getTransfers() != secondPath.getTransfers())
          {
            printf("\033[31m");
          }
          else
          {
            printf("\033[34m");
          }
          printf("DIFF of trips %d | %d (Dependent | Expanded ):\n"
                 "\033[0m", firstIndices[i], secondIndices[j]);
          printf("%s\n\n", firstPath.createDiffTo(secondPath).c_str());
        }
      }
      // if dependent found nothing, just print trips of expanded
      if (firstIndices.size() == 0)
      {
        for (size_t i = 0; i < secondIndices.size(); i++)
        {
          printf("\033[31mTrip %zu only found by Expanded:\033[0m\n", i);
          const TransitPath& secondPath = expandedPaths[secondIndices[i]];
          printf("%s\n\n", secondPath.toString().c_str());
        }
      }
    printf("---------------------------------------------------------------\n");
    printf("Dependent\n\n");
    for (size_t i = 0; i < dependentPaths.size(); i++)
    {
      printf("%s\n", dependentPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    printf("---------------------------------------------------------------\n");
    printf("Expanded\n\n");
    for (size_t i = 0; i < expandedPaths.size(); i++)
    {
      printf("%s\n", expandedPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    }
}
// _____________________________________________________________________________
int main(int argc, char** argv)
{
  string GTFSFolder;
  // Needed for getopt_long.
  int optChr;
  int numberOfQueries;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that GTFS folder was given
  if (argc - optind != 1)
  {
    printf("Please specify GTFS input folder!\n\n");
    printUsage();
    return 1;
  }
  // TODO(Robin): redo with one weekday only...
  char testDay = 0;
  int transferTime = 4 * 60;
  GTFSFolder = argv[optind];
  // parse transit network
  TransitNetworkDependent transitNetworkDependent;
  TransitNetworkExpanded transitNetworkExpanded;
  cout << "Parsing GTFS File by dependent network:" << endl;
  transitNetworkDependent.parseFromGTFSFiles(GTFSFolder);
  cout << " done." << endl;
  cout << "Parsing GTFS File by expanded network:" << endl;
  transitNetworkExpanded.setVerbosity(2);
  transitNetworkExpanded.parseFromGTFSFiles(GTFSFolder, testDay, transferTime);
  cout << " done." << endl;
  MulticriteriaDependentDijkstra routePlannerDependent(&transitNetworkDependent);
  DependentDijkstra routePlannerDependentPrecomputation(&transitNetworkDependent);
  MulticriteriaExpandedDijkstra routePlannerExpanded(&transitNetworkExpanded);
  // TODO(Robin): also allow direct use(?)...
  numberOfQueries = 100;
  //int startId, endId;
 // size_t settledNodes = 0;
  vector< vector <int> > randomInput;
  randomInput.resize(numberOfQueries);
  cout << "Creating of test data:" << endl;
  for (int i = 0; i < numberOfQueries; i++)
  {
    int startId = getRandomNodeId(transitNetworkDependent.getNumNodes());

    int endId = startId;
    while (startId == endId)
      endId = getRandomNodeId(transitNetworkDependent.getNumNodes());
    int hour = getRandomNodeId(24);
    int min = getRandomNodeId(60);
    int sec = getRandomNodeId(60);
    randomInput[i].push_back(startId); // 0
    randomInput[i].push_back(endId);   // 1
    randomInput[i].push_back(hour);    // 2
    randomInput[i].push_back(min);     // 3
    randomInput[i].push_back(sec);     // 4
  }
  /*
    cout << "EXPANDED:" << endl;
    routePlannerExpanded.computeShortestPath(11391, Time(0, 27, 16),
                                             8858, true);

    cout << "DEPENDED:" << endl;
    routePlannerDependent.computeShortestPath(11391, Time(0, 27, 16),
                                             8858, true);
    printf("---------------------------------------------------------------\n");
   return 0;*/
  cout << endl << "done." << endl;
  showDifferentPaths(routePlannerDependent.optimalPaths, routePlannerExpanded.optimalPaths);
  //return 0;
  cout << "Correctness testing of expanded and depandent dijkstra's:" << endl;
  int queriesWithDifferences = 0;
  for (int i = 0; i < numberOfQueries; i++)
  {
    bool resultsDifferent = false;
    cout << "----------------------------------------------------------------"
         << endl;
    //cout << startId << " " << endId << endl;
    //cout << day << ":" << hour <<":"<< min << endl;
    //WeekDate startTimeDep(0, randomInput[i][2], randomInput[i][3]);
    int startStopId = randomInput[i][0];
    int endStopId = randomInput[i][1];
    Time startTimeExp(randomInput[i][2], randomInput[i][3], randomInput[i][4]);
    cout << "EXPANDED:" << endl;
    routePlannerExpanded.computeShortestPath(startStopId, startTimeExp,
                                             endStopId, true);

    cout << "DEPENDED:" << endl;
    routePlannerDependentPrecomputation.computeShortestPath(startStopId, startTimeExp,
                                              endStopId, false);
    routePlannerDependent.timeMax = routePlannerDependentPrecomputation.time;
    routePlannerDependent.transferMax = routePlannerDependentPrecomputation.transfers;
    routePlannerDependent.computeShortestPath(startStopId, startTimeExp,
                                              endStopId, true);
    printf("---------------------------------------------------------------\n");
    // mark red in case there are differently many trips
    if (routePlannerDependent.optimalPaths.size() !=
        routePlannerExpanded.optimalPaths.size())
      printf("\033[31m");
      printf("Dependent %zu Paths, Expanded %zu Paths\n",
           routePlannerDependent.optimalPaths.size(),
           routePlannerExpanded.optimalPaths.size());
    if (routePlannerDependent.optimalPaths.size() !=
        routePlannerExpanded.optimalPaths.size())
      printf("\033[0m");
    // check if we found Different Paths, if yes diff everything
    // with everything
    pair<vector<int>, vector<int> > differentPaths =
                                            TransitPath::findDifferentResults(
                                            routePlannerDependent.optimalPaths,
                                            routePlannerExpanded.optimalPaths);
 
    if (differentPaths.first.size()> 0
        || differentPaths.second.size() > 0)
    {
      printf("\033[34mDIFFERENT PATHS:\033[0m\n");
      const vector<int>& firstIndices = differentPaths.first;
      const vector<int>& secondIndices = differentPaths.second;
      for (size_t i = 0; i < firstIndices.size(); i++)
      {
        // if expanded found nothing nothing, just print trips if dependent
        if (secondIndices.size() == 0)
        {
          printf("\033[31mTrip %zu only found by Dependent:\033[0m\n", i);
          printf("%s\n\n", routePlannerDependent.
                            optimalPaths[firstIndices[i]].toString().c_str());
          resultsDifferent = true;
        }
        for (size_t j = 0; j < secondIndices.size(); j++)
        {
          const TransitPath& firstPath = routePlannerDependent.
                                         optimalPaths[firstIndices[i]];
          const TransitPath& secondPath = routePlannerExpanded.
                                         optimalPaths[secondIndices[j]];
          // print inr ed if trips have different transfers or different
          // duration else just print blue
          if (firstPath.getDurationInMinutes() !=
              secondPath.getDurationInMinutes() ||
              firstPath.getTransfers() != secondPath.getTransfers())
          {
            printf("\033[31m");
            resultsDifferent = true;
          }
          else
          {
            printf("\033[34m");
          }
          printf("DIFF of trips %d | %d (Dependent | Expanded ):\n"
                 "\033[0m", firstIndices[i], secondIndices[j]);
          printf("%s\n\n", firstPath.createDiffTo(secondPath).c_str());
        }
      }
      // if dependent found nothing, just print trips of expanded
      if (firstIndices.size() == 0)
      {
        resultsDifferent = true;
        for (size_t i = 0; i < secondIndices.size(); i++)
        {
          printf("\033[31mTrip %zu only found by Expanded:\033[0m\n", i);
          const TransitPath& secondPath = routePlannerExpanded.
                                         optimalPaths[secondIndices[i]];
          printf("%s\n\n", secondPath.toString().c_str());
        }
      }
    printf("---------------------------------------------------------------\n");
    printf("Dependent\n\n");
    for (size_t i = 0; i < routePlannerDependent.optimalPaths.size(); i++)
    {
      printf("%s\n", routePlannerDependent.optimalPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    printf("---------------------------------------------------------------\n");
    printf("Expanded\n\n");
    for (size_t i = 0; i < routePlannerExpanded.optimalPaths.size(); i++)
    {
      printf("%s\n", routePlannerExpanded.optimalPaths[i].toString().c_str());
    }
    printf("---------------------------------------------------------------\n");
    }
    if (resultsDifferent)
      queriesWithDifferences++;
  }
  cout << endl << "done." << endl;
  cout << queriesWithDifferences << " different results." << endl;

  return 0;
}
