/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */
#include "./NodeExpanded.h"
#include <math.h>

NodeExpanded::NodeExpanded(int id, double latitude, double longitude)
{
  this->id = id;
  this->latitude = latitude;
  this->longitude = longitude;
  this->time = Time(0, 0, 0);
}

NodeExpanded::NodeExpanded(int id, double latitude, double longitude, int stopId,
        Time time, NodeType type)
{
  this->id = id;
  this->latitude = latitude;
  this->longitude = longitude;
  this->stopId = stopId;
  this->time = time;
  this->type = type;
}

double NodeExpanded::straightLineDistanceTo(const NodeExpanded& otherNode)
{
  // factor 111.111 to get kilometers...
  return 111.3 * sqrt(
               ((otherNode.latitude - this->latitude) *
               (otherNode.latitude - this->latitude))
              +
                ((otherNode.longitude - this->longitude) *
               (otherNode.longitude - this->longitude))
             );  //NOLINT , looks better this way no?
}

bool NodePointerTimeAndTypeComparator::operator()
                                        (const NodeExpanded* nodeA,
                                         const NodeExpanded* nodeB)
{
  if (nodeA->time < nodeB->time) return true;
  if (nodeB->time < nodeA->time) return false;
  // here time is equal, now sort by type
  // sorting arrival before transfer before departure...
  if (nodeA->type == nodeB->type)
  {
    // as last tie breaker for otherwise equal nodes, use their id...
    return nodeA->id < nodeB->id;
  }
  if (nodeA->type == ARRIVALNODE) return true;
  if (nodeB->type == ARRIVALNODE) return false;
  if (nodeA->type == TRANSFERNODE) return true;
  return false;
}

bool NodeTimeAndTypeComparator::operator()(const NodeExpanded& nodeA,
                                            const NodeExpanded& nodeB)
{
  if (nodeA.time < nodeB.time) return true;
  if (nodeB.time < nodeA.time) return false;
  // here time is equal, now sort by type
  // sorting arrival before transfer before departure...
  if (nodeA.type == nodeB.type)
  {
    // as last tie breaker for otherwise equal nodes, use their id...
    return nodeA.id < nodeB.id;
  }
  if (nodeA.type == ARRIVALNODE) return true;
  if (nodeB.type == ARRIVALNODE) return false;
  if (nodeA.type == TRANSFERNODE) return true;
  return false;
}
