/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 * Main Class for road network
 */
#include <getopt.h>
#include <string>
#include <vector>
#include "./RoadNetwork.h"
#include "./Dijkstra.h"
#include "./Timer.h"
void printUsage()
{
  printf("This program performs shortest path queries on road networks.\n"
         "USAGE: ./RoadNetworkMain [OPTIONS] osmFile\n"
         "Options are:\n"
         "--(q)ueries AMOUNT: number of queries that will be calculated\n"
         "--(h)elp: print this help and exit\n");
}

using std::string;
using std::vector;
int main(int argc, char** argv)
{
  string osmFileName;
  size_t numberOfQueries = 0;
  // Needed for getopt_long.
  int optChr;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"queries", required_argument, 0, 'q'},
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "hq:", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'q':
        numberOfQueries = atoi(optarg);
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that infileName and outFileName are entered
  if (argc - optind != 1)
  {
    printf("Please specify osm inputfile!\n\n");
    printUsage();
    return 1;
  }
  osmFileName = argv[optind];
  RoadNetwork roadNetwork;
  roadNetwork.readFromOsmFile(osmFileName);
  roadNetwork.reduceToBiggestComponent();
  StraightLineHeuristic aStarHeuristic;
  NoHeuristic noHeuristic;
  SimpleDijkstra routePlanner(&roadNetwork, &noHeuristic);
  vector<int> randomNodeIds;
  for (size_t i = 0; i < numberOfQueries; i++)
  {
    randomNodeIds.push_back(roadNetwork.getRandomNodeWithNeighbours()->id);
    randomNodeIds.push_back(roadNetwork.getRandomNodeWithNeighbours()->id);
  }
  Timer timer;
  int startId, endId;
  timer.start();
  size_t settledNodes = 0;
  for (size_t i = 0; i < numberOfQueries; i++)
  {
    startId = randomNodeIds[2 * i];
    endId = randomNodeIds[2 * i + 1];
    routePlanner.computeShortestPath(startId, endId, true);
    settledNodes += routePlanner.exploredNodes;
  }
  timer.stop();
  printf("Time taken for Dijkstra: %7.2f\n", timer.usecs() / 1000.0);
  printf("Average Time: %7.2f  Average Settled Nodes: %10zu\n",
          timer.usecs() / (1000.0 * numberOfQueries),
          settledNodes / (numberOfQueries));
}
