/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

#include <stdio.h>
#include <gtest/gtest.h>
#include "./RoadNetwork.h"


TEST(RoadNetworkTest, readOsmFile)
{
  FILE* testOsmFile;
  testOsmFile = fopen("testOSMFile.TEMP.xml", "w");
  fprintf(testOsmFile, "<?xml version=1.0>\n"
          "<osm version=\"0.6\" generator=\"pbf2osm\">\n>"
          "<node id=\"16\" lat=\"49.2\" lon=\"7.3\">\n"
          "</node>\n"
          "<node id=\"657\" lat=\"49.8\" lon=\"7.1\">\n"
          "</node>\n"
          "<node id=\"56\" lat=\"49.23\" lon=\"7.9\">\n"
          "</node>\n"
          "<node id=\"677\" lat=\"134.2\" lon=\"53.3\">\n"
          "</node>\n"
          "<node id=\"2334\" lat=\"34.2\" lon=\"8.3\">\n"
          "</node>\n"
          "<way id=\"3991523\">\n"
          "     <nd ref=\"16\"/>\n"
          "     <nd ref=\"677\"/>\n"
          "     <nd ref=\"56\"/>\n"
          "        <tag k=\"highway\" v=\"motorway\" />\n"
          "        <tag k=\"oneway\" v=\"yes\" />\n"
          "</way>\n"
          "<way id=\"3931523\">\n"
          "        <nd ref=\"2334\"/>\n"
          "        <nd ref=\"657\"/>\n"
          "        <tag k=\"highway\" v=\"motorway\" />\n"
          "</way>\n"
          "</osm>\n");

  fclose(testOsmFile);

  RoadNetwork roadNetwork;
  ASSERT_EQ("Nodes: \n"
            "Edges: ",
            roadNetwork.toString());
  roadNetwork.readFromOsmFile("testOSMFile.TEMP.xml");
  ASSERT_EQ("Nodes: 0[49.2/7.3], 1[49.8/7.1], 2[49.23/7.9], 3[134.2/53.3], 4[34.2/8.3]\n"
            "Edges: (0 -> 3 | 9.2569), (1 -> 4 | 1.49856), (3 -> 2 | 9.22715),"
            " (4 -> 1 | 1.49856)",
            roadNetwork.toString());
  remove("testOSMFile.TEMP.xml");
}


TEST(RoadNetworkTest, parseFromString)
{
  RoadNetwork roadNetwork;
  ASSERT_EQ("Nodes: \n"
           "Edges: ",
           roadNetwork.toString());
  roadNetwork.parseFromString(
           "Nodes: 0[13.0/14.1], 1[1/2], 2[0/0], 3[6/6], 4[1/1]\n"
           "Edges: (0 -> 3 | 878.626), (1 -> 4 | 142.237), (3 -> 2 | 875.802),"
           " (4 -> 1 | 142.237)");
  ASSERT_EQ("Nodes: 0[13/14.1], 1[1/2], 2[0/0], 3[6/6], 4[1/1]\n"
            "Edges: (0 -> 3 | 878.626), (1 -> 4 | 142.237), (3 -> 2 | 875.802),"
            " (4 -> 1 | 142.237)",
           roadNetwork.toString());
  roadNetwork.parseFromString("Nodes: 0[3.2/4.1], 1[0.3/2.2], 2[0.5/5.6]\n"
                              "Edges: (0 -> 2 | 10.564), (0 -> 1 | 3), "
                              "(2 -> 1 | 5.1)");
  ASSERT_EQ("Nodes: 0[3.2/4.1], 1[0.3/2.2], 2[0.5/5.6]\n"
            "Edges: (0 -> 2 | 10.564), (0 -> 1 | 3), "
            "(2 -> 1 | 5.1)",
            roadNetwork.toString());
}

TEST(RoadNetworkTest, reduceToBiggestComponent)
{
  {
    FILE* testOsmFile;
    testOsmFile = fopen("testOSMFile.TEMP.xml", "w");
    fprintf(testOsmFile, "<?xml version=1.0>\n"
            "<osm version=\"0.6\" generator=\"pbf2osm\">\n>"
            "<node id=\"16\" lat=\"49.2\" lon=\"7.3\">\n"
            "</node>\n"
            "<node id=\"657\" lat=\"49.8\" lon=\"7.1\">\n"
            "</node>\n"
            "<node id=\"56\" lat=\"49.23\" lon=\"7.9\">\n"
            "</node>\n"
            "<node id=\"677\" lat=\"134.2\" lon=\"53.3\">\n"
            "</node>\n"
            "<node id=\"2334\" lat=\"34.2\" lon=\"8.3\">\n"
            "</node>\n"
            "<way id=\"3991523\">\n"
            "     <nd ref=\"16\"/>\n"
            "     <nd ref=\"677\"/>\n"
            "     <nd ref=\"56\"/>\n"
            "        <tag k=\"highway\" v=\"motorway\" />\n"
            "        <tag k=\"oneway\" v=\"yes\" />\n"
            "</way>\n"
            "<way id=\"3931523\">\n"
            "        <nd ref=\"2334\"/>\n"
            "        <nd ref=\"657\"/>\n"
            "        <tag k=\"highway\" v=\"motorway\" />\n"
            "</way>\n"
            "</osm>\n");

    fclose(testOsmFile);

    RoadNetwork roadNetwork;
    ASSERT_EQ("Nodes: \n"
              "Edges: ",
              roadNetwork.toString());
    roadNetwork.readFromOsmFile("testOSMFile.TEMP.xml");
    ASSERT_EQ("Nodes: 0[49.2/7.3], 1[49.8/7.1], 2[49.23/7.9], 3[134.2/53.3], 4[34.2/8.3]\n"
              "Edges: (0 -> 3 | 9.2569), (1 -> 4 | 1.49856), "
              "(3 -> 2 | 9.22715), (4 -> 1 | 1.49856)",
              roadNetwork.toString());
    roadNetwork.reduceToBiggestComponent();
    ASSERT_EQ("Nodes: 0[49.2/7.3], 1[49.23/7.9], 2[134.2/53.3]\n"
              "Edges: (0 -> 2 | 9.2569), (2 -> 1 | 9.22715)",
              roadNetwork.toString());
    remove("testOSMFile.TEMP.xml");
  }
  {
    FILE* testOsmFile;
    testOsmFile = fopen("testOSMFile.TEMP.xml", "w");
    fprintf(testOsmFile, "<?xml version=1.0>\n"
            "<osm version=\"0.6\" generator=\"pbf2osm\">\n>"
            "<node id=\"16\" lat=\"49.2\" lon=\"7.3\">\n"
            "</node>\n"
            "<node id=\"657\" lat=\"49.8\" lon=\"7.1\">\n"
            "</node>\n"
            "<node id=\"56\" lat=\"49.23\" lon=\"7.9\">\n"
            "</node>\n"
            "<node id=\"677\" lat=\"89.2\" lon=\"53.3\">\n"
            "</node>\n"
            "<node id=\"2334\" lat=\"34.2\" lon=\"8.3\">\n"
            "</node>\n"
            "<node id=\"34544\" lat=\"34.2\" lon=\"8.3\">\n"
            "</node>\n"
            "<node id=\"444\" lat=\"34.2\" lon=\"8.3\">\n"
            "</node>\n"
            "<way id=\"3991523\">\n"
            "     <nd ref=\"16\"/>\n"
            "     <nd ref=\"677\"/>\n"
            "     <nd ref=\"56\"/>\n"
            "        <tag k=\"highway\" v=\"motorway\" />\n"
            "        <tag k=\"oneway\" v=\"yes\" />\n"
            "</way>\n"
            "<way id=\"3991523\">\n"
            "     <nd ref=\"677\"/>\n"
            "     <nd ref=\"444\"/>\n"
            "        <tag k=\"highway\" v=\"motorway\" />\n"
            "        <tag k=\"oneway\" v=\"yes\" />\n"
            "</way>\n"
            "<way id=\"3931523\">\n"
            "        <nd ref=\"2334\"/>\n"
            "        <nd ref=\"657\"/>\n"
            "        <tag k=\"highway\" v=\"motorway\" />\n"
            "</way>\n"
            "</osm>\n");

    fclose(testOsmFile);

    RoadNetwork roadNetwork;
    ASSERT_EQ("Nodes: \n"
              "Edges: ",
              roadNetwork.toString());
    roadNetwork.readFromOsmFile("testOSMFile.TEMP.xml");
    ASSERT_EQ("Nodes: 0[49.2/7.3], 1[49.8/7.1], 2[49.23/7.9], "
              "3[89.2/53.3], 4[34.2/8.3], 5[34.2/8.3], 6[34.2/8.3]\n"
              "Edges: (0 -> 3 | 5.83857), (1 -> 4 | 1.49856), (3 -> 2 | 5.79343),"
              " (3 -> 6 | 6.80635), (4 -> 1 | 1.49856)",
              roadNetwork.toString());
    roadNetwork.reduceToBiggestComponent();
    ASSERT_EQ("Nodes: 0[49.2/7.3], 1[49.23/7.9], 2[89.2/53.3], 3[34.2/8.3]\n"
              "Edges: (0 -> 2 | 5.83857), (2 -> 1 | 5.79343),"
              " (2 -> 3 | 6.80635)",
              roadNetwork.toString());
    remove("testOSMFile.TEMP.xml");
  }
}


TEST(RoadNetworkTest, getNearestNodesIdsSIMPLE)
{
  RoadNetwork roadNetwork;
  roadNetwork.parseFromString(
              "Nodes: 0[48.2/7.3], 1[49.8/7.1], 2[75.2/16.3], "
              "3[89.2/53.3], 4[34.2/8.3], 5[34.2/9.3]\n"
              "Edges: ");
  // Test 1: "Normal" case somewhere in the middle of these nodes
  double latitude = 44.8;
  double longitude = 8.0;
  int neighboursNumber = 3;
  int speedInKmh = 5;
  vector<Edge> nearestNodesAndDistances;
  roadNetwork.getNearestNodesIdsSIMPLE(latitude, longitude,
                                       neighboursNumber, speedInKmh,
                                       &nearestNodesAndDistances);
  ASSERT_EQ(neighboursNumber, nearestNodesAndDistances.size());
  for (size_t i = 0; i < nearestNodesAndDistances.size(); i++)
  {
  // Check that correct nodes were found.
    int nodeId = nearestNodesAndDistances[i].otherNodeId;
    double time = nearestNodesAndDistances[i].time;
    ASSERT_TRUE(nodeId == 0 || nodeId == 1 || nodeId == 4);
    // Check that times are correct
    NodeExpanded dummyNode(-1, latitude, longitude);
    double distance = dummyNode.straightLineDistanceTo(
                                *(roadNetwork.getNode(nodeId)));
    double correctTimeInSeconds = (distance * 60 * 60) /
                                  speedInKmh;
    ASSERT_TRUE(correctTimeInSeconds > 0);
    ASSERT_TRUE(time > 0);
    ASSERT_EQ(correctTimeInSeconds, time);
  }
    // Extreme cases test at one end and at the other end of the scale...
  latitude = 90.0;
  longitude = 180.0;
  nearestNodesAndDistances.clear();
  roadNetwork.getNearestNodesIdsSIMPLE(latitude, longitude,
                                       neighboursNumber, speedInKmh,
                                       &nearestNodesAndDistances);
  ASSERT_EQ(neighboursNumber, nearestNodesAndDistances.size());
  for (size_t i = 0; i < nearestNodesAndDistances.size(); i++)
  {
  // Check that correct nodes were found.
    int nodeId = nearestNodesAndDistances[i].otherNodeId;
    double time = nearestNodesAndDistances[i].time;
    ASSERT_TRUE(nodeId == 1 || nodeId == 2 || nodeId == 3);
    // Check that times are correct
    NodeExpanded dummyNode(-1, latitude, longitude);
    double distance = dummyNode.straightLineDistanceTo(
                                *(roadNetwork.getNode(nodeId)));
    double correctTimeInSeconds = (distance * 60 * 60) /
                                  speedInKmh;
    ASSERT_TRUE(correctTimeInSeconds > 0);
    ASSERT_TRUE(time > 0);
    ASSERT_EQ(correctTimeInSeconds, time);
  }
  latitude = -90.0;
  longitude = -180.0;
  nearestNodesAndDistances.clear();
  roadNetwork.getNearestNodesIdsSIMPLE(latitude, longitude,
                                       neighboursNumber, speedInKmh,
                                       &nearestNodesAndDistances);
  ASSERT_EQ(neighboursNumber, nearestNodesAndDistances.size());
  for (size_t i = 0; i < nearestNodesAndDistances.size(); i++)
  {
  // Check that correct nodes were found.
    int nodeId = nearestNodesAndDistances[i].otherNodeId;
    double time = nearestNodesAndDistances[i].time;
    ASSERT_TRUE(nodeId == 0 || nodeId == 4 || nodeId == 5);
    // Check that times are correct
    NodeExpanded dummyNode(-1, latitude, longitude);
    double distance = dummyNode.straightLineDistanceTo(
                                *(roadNetwork.getNode(nodeId)));
    double correctTimeInSeconds = (distance * 60 * 60) /
                                  speedInKmh;
    ASSERT_TRUE(correctTimeInSeconds > 0);
    ASSERT_TRUE(time > 0);
    ASSERT_EQ(correctTimeInSeconds, time);
  }
}
