/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 */

#include "./Time.h"
#include<stdio.h>
#include<assert.h>
#include <string>
#include <iomanip>
#include <iosfwd>
#include <sstream>
using std::flush;
using std::stringstream;

Time::Time()
{
  this->_secondsSinceDay = -1;
}
Time::Time(const Time& originalTime)
{
  // bypass checks because also invalid time is allowed to be copied
  this->_secondsSinceDay =  originalTime._secondsSinceDay;
}
Time::Time(char hour, char minutes, char seconds)
{
  // reset
  setSecondsSinceDay(0);
  assert(hour >= 0);
  assert(minutes >= 0);
  assert(seconds >= 0);
  setHour(hour);
  setMinute(minutes);
  setSecond(seconds);
}

Time::Time(int secondsSinceDay)
{
  setSecondsSinceDay(secondsSinceDay);
}
Time::Time(string GTFSTimeString)
{
  setSecondsSinceDay(0);
  // first comes hour, then colon, then minutes, then colon, then seconds
  char hour = atoi(GTFSTimeString.c_str());
  int posFirstColon = GTFSTimeString.find(':');
  char minutes = atoi(GTFSTimeString.c_str() + posFirstColon + 1);
  int posSecondColon = GTFSTimeString.find(':', posFirstColon + 1);
  char seconds = atoi(GTFSTimeString.c_str() + posSecondColon + 1);

  // for the case if number of hours is more as in one day
  if (hour > 23)
    hour = hour - 24;
  setHour(hour);
  setMinute(minutes);
  setSecond(seconds);
}
bool Time::operator< (const Time& otherTime) const
{
  return this->_secondsSinceDay < otherTime._secondsSinceDay;
}

bool Time::operator<= (const Time& otherTime) const
{
  return this->_secondsSinceDay <= otherTime._secondsSinceDay;
}

bool Time::operator== (const Time& otherTime) const
{
  return this->_secondsSinceDay == otherTime._secondsSinceDay;
}
int Time::minusInSeconds(const Time & otherTime) const
{
  return this->_secondsSinceDay - otherTime._secondsSinceDay;
}
int Time::minusInSecondsDependent(const Time & otherTime) const
{
  if (*this < otherTime)
    return 24 * 60 * 60 + this->_secondsSinceDay - otherTime._secondsSinceDay;
  else
    return this->_secondsSinceDay - otherTime._secondsSinceDay;
}
void Time::setHour(char hour)
{
  assert(_secondsSinceDay >= 0);
  assert(-1 < hour);
  this->setSecondsSinceDay(60 * 60 * hour + 60 * minute() + second());
  assert(_secondsSinceDay < 24 * 60 * 60);
}

void Time::setMinute(char minute)
{
  assert(_secondsSinceDay >= 0);
  assert(_secondsSinceDay < 24 * 60 * 60);
  assert(-1 < minute && minute < 60);
  this->setSecondsSinceDay(60 * 60 * hour() + 60 * minute + second());
  assert(_secondsSinceDay < 24 * 60 * 60);
}

void Time::setSecond(char second)
{
  assert(_secondsSinceDay >= 0);
  assert(_secondsSinceDay < 24 * 60 * 60);
  assert(-1 < second && second < 60);
  this->setSecondsSinceDay(60 * 60 * hour() + 60 * minute() + second);
}

void Time::setSecondsSinceDay(int secondsSinceDay)
{
  // check that seconds sicne day are in the range of 0:00 first day
  // till 23:59 next day
  assert(secondsSinceDay >= 0);
  assert(secondsSinceDay <  24 * 60 * 60);
  _secondsSinceDay = secondsSinceDay;
}


void Time::addSeconds(int seconds)
{
  assert(seconds >= 0);
  // make sure seconds are within one day (less than 24 hours)...
  int newSecondsSinceDay = _secondsSinceDay + seconds;
  while (newSecondsSinceDay >= 24 * 60 * 60)
    newSecondsSinceDay -= 24 *60 *60;
  this->setSecondsSinceDay(newSecondsSinceDay);
}

int Time::hour() const { return _secondsSinceDay / (60 * 60);}
int Time::minute() const
{
  int secondsThisHour = static_cast<int>(_secondsSinceDay) % 3600;
  int minutesThisHour = secondsThisHour / 60;
  return minutesThisHour;
}
int Time::second() const
{
  int secondsThisMinute = static_cast<int>(_secondsSinceDay) % 60;
  return secondsThisMinute;
}

string Time::toString() const
{
  stringstream resultStream;
  resultStream << *this;
  return resultStream.str();
}
void Time::printTime() const
{
  std::cout << *this << "\n";
}

string Time::createJSON() const
{
    stringstream jsonStringStream;
  // add duration, transfers and nodes
  jsonStringStream << "{\n"
                   << "\"hour\": " << static_cast<int>(hour()) << ",\n"
                   << "\"minute\": " << static_cast<int>(minute()) << ",\n"
                   << "\"second\": " << static_cast<int>(second()) << "\n"
                   << "}\n";
  return jsonStringStream.str();
}
int Time::secondsSinceDay() const {
  return _secondsSinceDay;
}


ostream& operator<< (ostream& outStream, const Time& time)
{
  // output time like this: 05:24:09 (always two digits per number)
  outStream << std::setw(2) <<std::setfill('0')
            << static_cast<int>(time.hour()) << ":"
            << std::setw(2) <<std::setfill('0')
            << static_cast<int>(time.minute()) << ":"
            << std::setw(2) <<std::setfill('0')
            << static_cast<int>(time.second());
  return outStream;
}
