/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 */
#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include "./TransitNetworkExpanded.h"
#include "./Dijkstra.h"
#include "./Timer.h"
#include "./TransferPatterns.h"

void printUsage()
{
  printf("This program performs shortest path queries on transit networks.\n\n"
         "USAGE: ./TransitNetworkMain [OPTIONS] GTFSFolder\n");
}

using std::string;
using std::vector;
int main(int argc, char** argv)
{
  string GTFSFolder;
  // Needed for getopt_long.
  int optChr;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }
  // check that GTFS folder was given
  if (argc - optind != 1)
  {
    printf("Please specify GTFS input folder!\n\n");
    printUsage();
    return 1;
  }
  GTFSFolder = argv[optind];
  // parse transit network for one day
  char validWeekday = 0;
  int transferTimeInSeconds = 4 * 60;
  TransitNetworkExpanded transitNetwork;
  transitNetwork.parseFromGTFSFiles(GTFSFolder, validWeekday,
                                    transferTimeInSeconds);
  TransferPatterns routePlanner(&transitNetwork, new NoHeuristic());
  printf("Nodes in Transit Network: %zu\n", transitNetwork.getNumNodes());
  printf("Stops in Transit Network: %zu\n", transitNetwork.getNumStops());
  // TODO(Robin): also allow direct use(?)...
  Timer timer;
  timer.start();
  routePlanner.doPrecomputation();
  timer.stop();
  printf("Time taken for computing of TransferPatterns: %7.2f\n",
          timer.usecs() / 1000.0);
}