/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 */


#include "./TransitNetworkDependent.h"
#include <unordered_map>
#include <unordered_set>
#include <string>
#include <utility>
#include <vector>
#include <algorithm>
#include <limits>
#include <functional>
#include <sstream>
#include "./CsvParser.h"
#include "./TransitParser.h"

using std::string;
using std::cout;
using std::endl;
using std::unordered_map;
using std::pair;
using std::vector;
using std::unordered_set;

TransitNetworkDependent::TripFrequency::TripFrequency()
{
      intervalInSeconds = -1;
      startHour = -1;
      startMinute = -1;
      startSecond = -1;
      endHour = -1;
      endMinute = -1;
      endSecond = -1;
}

TransitNetworkDependent::TripFrequency::TripFrequency(char startHour, char startMinute,
                                             char startSecond,
                                             char endHour, char endMinute,
                                             char endSecond,
                                             int intervalInSeconds)
{
      this->startHour = startHour;
      this->startMinute = startMinute;
      this->startSecond = startSecond;
      this->endHour = endHour;
      this->endMinute = endMinute;
      this->endSecond = endSecond;
      this->intervalInSeconds = intervalInSeconds;
}

TransitNetworkDependent::TransitNetworkDependent()
{
  this->_outgoingEdges = new vector<vector<EdgeDependent> >();
  this->_incomingEdges = new vector<vector<EdgeDependent> >();
  this->_nodes = new vector<NodeDependent>();
  this->_stopIdNameToId = new unordered_map<string, int> ();
}

TransitNetworkDependent::TransitNetworkDependent(const TransitNetworkDependent& orig) : Graph(orig)
{
  *(this->_outgoingEdges) = *(orig._outgoingEdges);
  *(this->_incomingEdges) = *(orig._incomingEdges);
  *(this->_nodes) = *(orig._nodes);
  *(this->_stopIdNameToId) = *(orig._stopIdNameToId);
}

TransitNetworkDependent::~TransitNetworkDependent()
{
  if (_outgoingEdges != NULL)
    delete _outgoingEdges;
  _outgoingEdges = NULL;
  if (_incomingEdges != NULL)
    delete _incomingEdges;
  _incomingEdges = NULL;
  if (_nodes != NULL)
    delete _nodes;
  _nodes = NULL;
  if (_stopIdNameToId != NULL)
    delete _stopIdNameToId;
  _stopIdNameToId = NULL;
}


// _____________________________________________________________________________
void TransitNetworkDependent::parseFromGTFSFiles(string folderName)
{
  // TODO(Robin): make weeday and transfer timea rguments of the function..
  char validWeekday = 0;
  // parse which service ids serve which weekdays
  unordered_set<string> validServiceIds;
  printf("parsing %s ...\n", (folderName + "/calendar.txt").c_str());
  parseCalendarFile(folderName + "/calendar.txt", validWeekday,
                    &validServiceIds);
  // parse which trips serve which weekdays
  unordered_map<string, int > tripStrToIntId;
  printf("parsing %s ...\n", (folderName + "/trips.txt").c_str());
  parseTripsFile(folderName + "/trips.txt", validServiceIds, &tripStrToIntId);
  // parse which stop id name string belongs to which stop id number
  // and to which coordinates
  //unordered_map<string, int> stopIdNameToId;
  printf("parsing %s ...\n", (folderName + "/stops.txt").c_str());
  parseStopsFile(folderName + "/stops.txt");
  // parse which which trip id names have which frequency
  unordered_map<string, vector<TransitParser::TripFrequency> >
                                                              tripIdToFrequency;
  printf("parsing %s ...\n", (folderName + "/frequencies.txt").c_str());
  parseFrequenciesFile(folderName + "/frequencies.txt", &tripIdToFrequency);
  // parse stops themselves, create arrival departure and transfer nodes
  // also consider frequencies..
  printf("parsing %s ...\n", (folderName + "/stop_times.txt").c_str());
  parseStopTimesFile(folderName + "/stop_times.txt",
                     tripIdToFrequency,
                     tripStrToIntId);
  // Build map tripId to vector of ConectionId, wich belong to it,
  // by outgoingEdges
  for (size_t i = 0; i < _outgoingEdges->size(); i++)
  {
    for (size_t j = 0; j < _outgoingEdges->at(i).size(); j++)
    {
      _outgoingEdges->at(i)[j].BuildTripsMap();
    }
  }

  // Build map tripId to vector of ConectionId, wich belong to it,
  // by incomingEdges
  for (size_t i = 0; i < _incomingEdges->size(); i++)
  {
    for (size_t j = 0; j < _incomingEdges->at(i).size(); j++)
    {
      _incomingEdges->at(i)[j].BuildTripsMap();
    }
  }
  printf("...done parsing.\n");
}
// _____________________________________________________________________________
void TransitNetworkDependent::parseCalendarFile(const string& calendarFileName,
                          char validWeekDay,
                          unordered_set<string>* validServiceIds)
{
  int verbosity = 2;
  TransitParser parser(verbosity);
  parser.parseCalendarFile(calendarFileName, validWeekDay, validServiceIds);
}
// _____________________________________________________________________________
void TransitNetworkDependent::parseTripsFile(string tripFileName,
                                 const unordered_set<string>& validServiceIds,
                                 unordered_map<string, int>* tripIdNameToTripId)
{
  int verbosity = 2;
  TransitParser parser(verbosity);
  parser.parseTripsFile(tripFileName, validServiceIds, tripIdNameToTripId);
}
// _____________________________________________________________________________
void TransitNetworkDependent::parseStopsFile(string stopFileName)
{  
  int verbosity = 2;
  TransitParser parser(verbosity);
  vector<Stop> parsedStops;
  parser.parseStopsFile(stopFileName, &parsedStops , _stopIdNameToId);

  for (size_t i = 0; i < parsedStops.size(); i++)
  {
    const Stop& stop = parsedStops[i];
    // Create Node for the stop
    NodeDependent node(stop.stopId, stop.latitude, stop.longitude);
    _nodes->push_back(node);
    assert(static_cast<int>(i) == stop.stopId);
    _stopIdToName.push_back( stop.stopName);
  }
  _incomingEdges->resize(_nodes->size());
  _outgoingEdges->resize(_nodes->size());
}
// _____________________________________________________________________________
void TransitNetworkDependent::parseFrequenciesFile(string frequencyFileName,
                  unordered_map<string, vector<TransitParser::TripFrequency> >*
                  tripIdNameToFrequencies)
{

  int verbosity = 2;
  TransitParser parser(verbosity);
  parser.parseFrequenciesFile(frequencyFileName, tripIdNameToFrequencies);
}

// _____________________________________________________________________________
void TransitNetworkDependent::parseStopTimesFile(string stopTimesFileName,
        const unordered_map<string, vector<TransitParser::TripFrequency> >&
                                          tripIdNameToFrequencies,
        const unordered_map<string, int>& tripStrToIntId)
{
  vector<TransitParser::Trip> parsedTrips;

  int verbosity = 2;
  TransitParser parser(verbosity);
  int transferTime = 4 * 60;
  parser.parseStopTimesFile(stopTimesFileName, *_stopIdNameToId,
           tripIdNameToFrequencies, tripStrToIntId,transferTime, &parsedTrips);

  // now add all edges...
  for (size_t i = 0; i < parsedTrips.size(); i++)
  {
    const vector<TransitParser::TripFrequency>& currentFrequencies =
      parsedTrips[i].frequencies;
    const vector<TransitParser::TripNode>& tripNodes = parsedTrips[i].nodes;
    int tripId =  parsedTrips[i].tripId;
    for (size_t j = 2; j < tripNodes.size() - 2; j += 3)
    {
      // first comes departure node, then arrival node, ignore transfer node
      // after arrival node..
      int departureSecondsSinceDay = tripNodes[j].secondsSinceDay;
      if (departureSecondsSinceDay >= 24 * 60 * 60)
        departureSecondsSinceDay -= 24 * 60 * 60;
      int arrivalSecondsSinceDay = tripNodes[j + 1].secondsSinceDay;
      if (arrivalSecondsSinceDay >= 24 * 60 * 60)
        arrivalSecondsSinceDay -= 24 * 60 * 60;
      Time departureTime(departureSecondsSinceDay);
      Time arrivalTime(arrivalSecondsSinceDay);
      // add for all frequencies..
      for (size_t k = 0; k < currentFrequencies.size(); k++)
      {
        addTrip(tripNodes[j].stopId, tripNodes[j + 1].stopId,
               currentFrequencies[k],
               departureTime, arrivalTime, tripId);
      }
    }
  }
  printf("  done.\n");
//  size_t sum=0;
//  size_t num=0;
//  for (int i = 0; i < _outgoingEdges->size(); i++)
//    for (int j = 0; j < (*_outgoingEdges)[i].size(); j++)
//    {
//      cout <<"\t"<< i << "\t" << j << "\t" << (*_outgoingEdges)[i][j]._times.size() << endl;
//      num++;
//      sum += (*_outgoingEdges)[i][j]._times.size();
//    }
//  cout << "num=" << num <<" sum=" << sum << " schnitt="<< sum/num;
}
// _____________________________________________________________________________
// TODO(Robin): switch creation of transfer nodes from parsestoptimesfile
// to this function?
void TransitNetworkDependent::addTrip(int startNodeId, int endNodeId,
                          const TransitParser::TripFrequency& frequency,
                          Time startTime, Time endTime,
                          int tripId)
{
    // make trip for this weekday....
    int startSecondInDay = frequency.startHour * 60 * 60 +
                           frequency.startMinute * 60 +
                           frequency.startSecond;
     int endSecondInDay =  frequency.endHour * 60 * 60 +
                           frequency.endMinute * 60 +
                           frequency.endSecond;
     int durationInSeconds = endSecondInDay - startSecondInDay;
     int offsetInSeconds = 0;
    // always have to finish first trip, this is here
    // in case trip has no frequency..
    bool tripFinished = false;
    while (!tripFinished)
    {
      int startSecondsSinceDay = startTime.secondsSinceDay()
                               + offsetInSeconds;
      if (startSecondsSinceDay >= 24 * 60 * 60)
        startSecondsSinceDay = startSecondsSinceDay - 24 * 60 * 60;

      int endSecondsSinceDay = endTime.secondsSinceDay()
                             + offsetInSeconds;
      if (endSecondsSinceDay >= 24 * 60 * 60)
        endSecondsSinceDay = endSecondsSinceDay - 24 * 60 * 60;

      Time newStartTime(startSecondsSinceDay);
      Time newEndTime(endSecondsSinceDay);

      addStopTimeToCorrespondEdge(startNodeId, endNodeId,
              newStartTime, newEndTime,
              tripId);

      offsetInSeconds += frequency.intervalInSeconds;
      // if no real frequency given, one trip is enough...
      if (frequency.intervalInSeconds == -1)
        tripFinished = true;
      else
        tripFinished =  offsetInSeconds > durationInSeconds;
    }
}


// _____________________________________________________________________________
// So we try here to add a conection(StopTime) to corresponding edge
// if the edge at this time doesn't exist, than create an edge between given
// Nodes(Stops) and add inside given conection
void TransitNetworkDependent::addStopTimeToCorrespondEdge(int startStopId,
                                                          int endStopId,
                                                          Time startTime,
                                                          Time endTime,
                                                          int lineId)
{
  size_t i = 0;
  bool flag = true;
  while (i < _outgoingEdges->at(startStopId).size() && flag)
  {
    if ((*_outgoingEdges)[startStopId][i].otherNodeId == endStopId)
    {
      flag = false;
//      if (lineId == 4061)
//        cout << "start " << startStopId << "end" << endStopId << endl;
      (*_outgoingEdges)[startStopId][i].AddStopTime(startTime, endTime, lineId);
      break;
    }
    i++;
  }
  if (i == _outgoingEdges->at(startStopId).size() && flag)
  {
    addEdge(startStopId, endStopId);
//    if (lineId == 4061)
//        cout << "start " << startStopId << "end" << endStopId << endl;
    (*_outgoingEdges)[startStopId][i].AddStopTime(startTime, endTime, lineId);
  }
  i = 0;
  flag = true;
  while (i < _incomingEdges->at(endStopId).size() && flag)
  {
    if ((*_incomingEdges)[endStopId][i].otherNodeId == startStopId)
    {
      flag = false;
      (*_incomingEdges)[endStopId][i].AddStopTime(startTime, endTime, lineId);
      break;
    }
    i++;
  }
  /*if (i == _incomingEdges->at(endStopId).size())
  {
    addEdge(endStopId, endStopId);
    (*_incomingEdges)[endStopId][i].AddStopTime(startTime, endTime, lineId);
  }*/
}
// _____________________________________________________________________________
void TransitNetworkDependent::addEdge(int startId, int endId)
{
  if (static_cast<size_t>(startId) >= _nodes->size())
  {
    printf("Error: startId %d outside of range (graph has %zu nodes)\n",
            startId, _nodes->size());
  }
  if (static_cast<size_t>(endId) >= _nodes->size())
  {
    printf("Error endId %d outside of range (graph has %zu nodes)\n",
            endId, _nodes->size());
  }
  if (static_cast<size_t>(startId) >= _outgoingEdges->size())
  {
    printf("Error startid %d outside of outedge range (adjacency lists have "
            "%zu lists)\n",
            startId, _outgoingEdges->size());
  }
  if (static_cast<size_t>(endId) >= _incomingEdges->size())
  {
    printf("Error endid %d outside of inedge range (adjacency lists have "
            "%zu lists)\n",
            endId, _incomingEdges->size());
  }
  _outgoingEdges->at(startId).push_back(EdgeDependent(endId));
  _incomingEdges->at(endId).push_back(EdgeDependent(startId));
}
// _____________________________________________________________________________
NodeDependent* TransitNetworkDependent::getNode(int id) const
{
  return &((*_nodes)[id]);
}
// _____________________________________________________________________________
vector<EdgeDependent>* TransitNetworkDependent::getOutEdges(int nodeId) const
{
  return &((*_outgoingEdges)[nodeId]);
}
// _____________________________________________________________________________
void TransitNetworkDependent::getStopsStartingWith(const string& prefix,
                                          vector<string>* stopNames) const
{
  size_t prefixLength = prefix.size();
  for (size_t i = 0; i < _stopIdToName.size(); i++)
  {

    string stopName = _stopIdToName[i];
    string stopPrefix = stopName.substr(0, prefixLength);
    // compare case insensitive
    if (strcasecmp(stopPrefix.c_str(), prefix.c_str()) == 0)
      stopNames->push_back(stopName);
  }
}
// _____________________________________________________________________________
string TransitNetworkDependent::getStopName(size_t id)
{
  if (id < 0 || id >= _stopIdToName.size())
    return "noname";
  return _stopIdToName[id];
}
// _____________________________________________________________________________
size_t TransitNetworkDependent::getNumNodes() const
{
  return _nodes->size();
}
// _____________________________________________________________________________
size_t TransitNetworkDependent::getNumStops() const
{
  return _nodes->size();
}
// _____________________________________________________________________________
int TransitNetworkDependent::getNodeIdByTheStopName(string stopName)
{
  for (size_t i = 0; i < _stopIdToName.size(); i++)
  {
    if (_stopIdToName[i] == stopName)
      return i;
  }
  return -1;
}
// _____________________________________________________________________________

void TransitNetworkDependent::getConnections(int nodeId, Time arrivalTime,
                                             int tripId,
                                         bool isMultiCriteria, bool isStartNode,
                                         vector<LiveConnection>* outConnections)
{
  
    // get outgoing edges from this node
  vector<EdgeDependent>* edges = this->getOutEdges(nodeId);
  // compute all new distances in loop
  int newId;
  int timeNeededForConnection;
  char transfers;
  for (size_t i = 0; i < edges->size(); i++)
  {
    // get all new distances
    EdgeDependent& newEdge = (*edges)[i];
    newId = newEdge.otherNodeId;
    vector<Connection> newStopTimes;
    //TODO(RObin): instead make edge weight accept time and line id directly...
    Connection currentConnection;
    currentConnection.arrivalTime = arrivalTime;
    currentConnection.lineId = tripId;
    bool tripContinues = true;
    // TODO(RObin): also change this somehow... make it more adaptable
    int transferTime = 4 * 60;
    if (isStartNode)
      transferTime = 0;
    newEdge.GetWight(currentConnection, transferTime,
                     &newStopTimes, isStartNode, isMultiCriteria,
                     &tripContinues);
    if (newStopTimes.size() == 0)
      continue;
//      cout << "*** " << newStopTimes.size()<< endl;
    for (size_t j = 0; j < newStopTimes.size(); j++)
    {
//        cout << "---" << endl;
//        newStopTimes[i].startTime.printTime();
//        newStopTimes[i].endTime.printTime();
      if ((tripId != newStopTimes[j].lineId &&
               (arrivalTime.secondsSinceDay() + transferTime >
                newStopTimes[j].departureTime.secondsSinceDay()
              && arrivalTime.secondsSinceDay() <=
                newStopTimes[j].arrivalTime.secondsSinceDay()))
         ||(tripId == newStopTimes[j].lineId &&
               (arrivalTime.secondsSinceDay() >
                newStopTimes[j].departureTime.secondsSinceDay()
              && arrivalTime.secondsSinceDay() <=
                newStopTimes[j].arrivalTime.secondsSinceDay())))
      {
        timeNeededForConnection = 24 * 60 * 60 + newStopTimes[j]
                .arrivalTime.minusInSecondsDependent(arrivalTime);
      }
      else
      {
        timeNeededForConnection = newStopTimes[j]
                .arrivalTime.minusInSecondsDependent(arrivalTime);
      }
      if (tripId != newStopTimes[j].lineId && !isStartNode)
        transfers = 1;
      else
        transfers = 0;
      bool endOfTrip = !tripContinues;
      LiveConnection newConnection(newStopTimes[j], TRANSITNETWORK,
                                   newId,
                                   timeNeededForConnection,
                                   transfers, endOfTrip);
      outConnections->push_back(newConnection);
    }
  }
}