/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 */
#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include "./EdgeDependent.h"
#include "./TransitNetworkDependent.h"
#include "./Timer.h"
#include "./MulticriteriaDependentDijkstra.h"

void printUsage()
{
  printf("This program performs shortest path queries on dependent "
         "transit networks.\n\n"
         "USAGE: ./TransitNetworkDependentMain [OPTIONS] GTFSFolder\n");
}

using std::string;
using std::vector;
int main(int argc, char** argv)
{
  string GTFSFolder;
  // Needed for getopt_long.
  int optChr;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that GTFS folder was given
  if (argc - optind != 1)
  {
    printf("Please specify GTFS input folder!\n\n");
    printUsage();
    return 1;
  }
  GTFSFolder = argv[optind];
  // parse transit network
  TransitNetworkDependent transitNetwork;
  transitNetwork.parseFromGTFSFiles(GTFSFolder);
  MulticriteriaDependentDijkstra routePlanner(&transitNetwork);
  printf("Nodes in Transit Network: %zu\n", transitNetwork.getNumNodes());
  // TODO(Robin): also allow direct use(?)...
  while (true)
  {
    printf("Please enter the beginning letters for the departure stop: ");
    string stopPrefix;
    vector<string> possibleStops;
    while (possibleStops.size() == 0)
    {
      std::cin >> stopPrefix;
      transitNetwork.getStopsStartingWith(stopPrefix, &possibleStops);
      if (possibleStops.size() == 0)
        printf("No stops found starting with %s\n", stopPrefix.c_str());
    }
    printf("\nChoose your departure stop:\n");
    for (size_t i = 0; i < possibleStops.size(); i++)
    {
      printf("%3zu - %s\n", i, possibleStops[i].c_str());
    }
    int departureStopNr;
    std::cin >> departureStopNr;
    string departureStopName = possibleStops[departureStopNr];
    printf("Please enter the beginning letters for the destination stop: ");
    possibleStops.clear();
    stopPrefix = "";
    while (possibleStops.size() == 0)
    {
      std::cin >> stopPrefix;
      transitNetwork.getStopsStartingWith(stopPrefix, &possibleStops);
      if (possibleStops.size() == 0)
        printf("No stops found starting with %s\n", stopPrefix.c_str());
    }
    printf("\nChoose your destination stop:\n");
    for (size_t i = 0; i < possibleStops.size(); i++)
    {
      printf("%3zu - %s\n", i, possibleStops[i].c_str());
    }
    int destinationStopNr;
    std::cin >> destinationStopNr;
    string destinationStopName = possibleStops[destinationStopNr];

    string input;
    printf("\nPlease enter the hour of departure\n");
    std::cin >> input;
    char hour = atoi(input.c_str());

    if (hour < 0 || hour > 23)
      printf("Please enter an hour between 0 and 23\n");

    printf("\nPlease enter the minute of departure\n");
    std::cin >> input;
    char minute = atoi(input.c_str());
    if (minute < 0 || minute > 59)
      printf("Please enter a minute between 0 and 59\n");
    Time startTime(hour, minute, 0);
    printf("computing path...\n");
    routePlanner.computeShortestPath(transitNetwork.
                                   getNodeIdByTheStopName(departureStopName),
                                     startTime,
                                     transitNetwork.
                                   getNodeIdByTheStopName(destinationStopName)
                                    ,true);
    printf("Path: \n");
    for (size_t i = 0; i < routePlanner.transitShortestPath.size(); i++)
    {
      std::cout << routePlanner.transitShortestPath[i] << std::endl;
    }
    printf("Enter q to quit, other key to continue\n");
    std::cin >> input;
    if (input == "q" || input == "Q")
      return 0;
  }
//  WeekDate startTime(0, 23, 59);
//    printf("computing path...\n");
//    routePlanner.computeShortestPath(0, startTime,
//                                     4,true);
//    printf("Path: \n");
//    for (size_t i = 0; i < routePlanner.transitShortestPath.size(); i++)
//    {
//      std::cout << routePlanner.transitShortestPath[i] << std::endl;
//    }
  return 0;
}