#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include "./Timer.h"
#include "./Dijkstra.h"
#include "./EdgeDependent.h"
#include "./TransitNetworkDependent.h"
#include "./DependentDijkstra.h"
#include "./ExpandedDijkstra.h"

using std::string;
using std::vector;
using std::cout;
using std::endl;
unsigned int randomSeedPerf;
// _____________________________________________________________________________
void printUsage()
{
  printf("This program measures performance for shortest path queries "
         "transit networks.\n\n"
         "USAGE: ./TransitNetworkDependentPerf GTFSFolder\n");
}
// _____________________________________________________________________________
int getRandomNodeId(int size)
{
  int idToTry = rand_r(&randomSeedPerf) % size;
  return idToTry;
}
// _____________________________________________________________________________
int main(int argc, char** argv)
{
  string GTFSFolder;
  // Needed for getopt_long.
  int optChr;
  int numberOfQueries;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that GTFS folder was given
  if (argc - optind != 1)
  {
    printf("Please specify GTFS input folder!\n\n");
    printUsage();
    return 1;
  }
  // TODO(Robin): redo with one weekday only...
  char testDay = 0;
  int transferTime = 4 * 60;
  GTFSFolder = argv[optind];
  // parse transit network
  TransitNetworkDependent transitNetworkDependent;
  TransitNetworkExpanded transitNetworkExpanded;
  cout << "Parsing GTFS File by dependent network:" << endl;
  transitNetworkDependent.parseFromGTFSFiles(GTFSFolder);
  cout << " done." << endl;
  cout << "Parsing GTFS File by expanded network:" << endl;
  transitNetworkExpanded.parseFromGTFSFiles(GTFSFolder, testDay, transferTime);
  cout << " done." << endl;
  DependentDijkstra routePlannerDependent(&transitNetworkDependent);
  ExpandedDijkstra routePlannerExpanded(&transitNetworkExpanded);
  // TODO(Robin): also allow direct use(?)...
  numberOfQueries = 100;
  //int startId, endId;
 // size_t settledNodes = 0;
  vector< vector <int> > randomInput;
  randomInput.resize(numberOfQueries);
  cout << "Creating of test data:" << endl;
  for (int i = 0; i < numberOfQueries; i++)
  {
    int startId = getRandomNodeId(transitNetworkDependent.getNumNodes());

    int endId = startId;
    while (startId == endId)
      endId = getRandomNodeId(transitNetworkDependent.getNumNodes());
    int hour = getRandomNodeId(24);
    int min = getRandomNodeId(60);
    int sec = getRandomNodeId(60);
    randomInput[i].push_back(startId); // 0
    randomInput[i].push_back(endId);   // 1
    randomInput[i].push_back(hour);    // 2
    randomInput[i].push_back(min);     // 3
    randomInput[i].push_back(sec);     // 4
  }

  cout << endl << "done." << endl;
  cout << "Correctness testing of expanded and depandent dijkstra's:" << endl;
  for (int i = 0; i < numberOfQueries; i++)
  {
    cout << "----------------------------------------------------------------"
         << endl;
    //cout << startId << " " << endId << endl;
    //cout << day << ":" << hour <<":"<< min << endl;
    //WeekDate startTimeDep(0, randomInput[i][2], randomInput[i][3]);
    int startStopId = randomInput[i][0];
    int endStopId = randomInput[i][1];
    Time startTimeExp(randomInput[i][2], randomInput[i][3], randomInput[i][4]);
    cout << "EXPANDED:" << endl;
    routePlannerExpanded.computeShortestPath(startStopId, startTimeExp,
                                             endStopId, true);
    for (size_t j = 0; j < routePlannerExpanded.transitShortestPath.size(); j++)
    {
      NodeExpanded node = routePlannerExpanded.transitShortestPath.at(j).first;
      Edge edge = routePlannerExpanded.transitShortestPath.at(j).second;
      if (node.type == DEPARTURENODE || (j > 0  && node.type == ARRIVALNODE))
      {
        printf("%-10s: %-22s Time: %02d:%02d:%02d (%d)\n",
                node.type == ARRIVALNODE ? "Arrival" : "Departure",
                              transitNetworkExpanded.getStopName(node.stopId).c_str(),
                                         node.time.hour(), node.time.minute(), node.time.second(),
                                         transitNetworkExpanded.getTripId(node.id));//node.time.getDayName().c_str());
      }
    }
    cout << "DEPENDED:" << endl;
    routePlannerDependent.computeShortestPath(randomInput[i][0], startTimeExp,
                                              randomInput[i][1], true);
    for (size_t j = 0; j < routePlannerDependent.transitShortestPath.size(); j++)
    {
      cout << routePlannerDependent.transitShortestPath[j] << std::endl;
    }
    cout << "----------------------------------------------------------------"
         << endl;
  }
  cout << endl << "done." << endl;
  return 0;
}
